@@ -31,26 +31,22 @@
 
 #define ADIF_MAX_SIZE 40 /* Should be enough */
 #define ADTS_MAX_SIZE 10 /* Should be enough */
+static const int aac_sample_rates[] = {
+  SAMPLE_RATE_96KHZ,
+  SAMPLE_RATE_88_2KHZ,
+  SAMPLE_RATE_64KHZ,
+  SAMPLE_RATE_48KHZ,
+  SAMPLE_RATE_44_1KHZ,
+  SAMPLE_RATE_32KHZ,
+  SAMPLE_RATE_24KHZ,
+  SAMPLE_RATE_22_05KHZ,
+  SAMPLE_RATE_16KHZ,
+  SAMPLE_RATE_12KHZ,
+  SAMPLE_RATE_11_025KHZ,
+  SAMPLE_RATE_8KHZ
+};
 
-static int aac_sample_rates[] = { SAMPLE_RATE_96KHZ,
-                                  SAMPLE_RATE_88_2KHZ,
-                                  SAMPLE_RATE_64KHZ,
-                                  SAMPLE_RATE_48KHZ,
-                                  SAMPLE_RATE_44_1KHZ,
-                                  SAMPLE_RATE_32KHZ,
-                                  SAMPLE_RATE_24KHZ,
-                                  SAMPLE_RATE_22_05KHZ,
-                                  SAMPLE_RATE_16KHZ,
-                                  SAMPLE_RATE_12KHZ,
-                                  SAMPLE_RATE_11_025KHZ,
-                                  SAMPLE_RATE_8KHZ };
-
-GstElementDetails gst_dspaacsink_details = GST_ELEMENT_DETAILS ("DSP AAC Sink",
-    "Sink/Audio",
-    "AAC audio sink",
-    "Makoto Sugano <makoto.sugano@nokia.com>");
-
-static gchar *devname = "/dev/dsptask/aacdec";
+static const gchar *devname = "/dev/dsptask/aacdec";
 
 static GstStaticPadTemplate dspaacsink_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -87,36 +83,35 @@ static GstFlowReturn gst_dspaacsink_preroll (GstBaseSink *sink, GstBuffer * buff
 static gboolean gst_dspaacsink_event2 (GstBaseSink *sink, GstEvent *event);
 
 
-/**
- *
- *
- */
-
-void _do_init()
+#ifdef DEBUG
+static void _do_init()
 {
   DBG_PRINT("AAC DO_INIT\n");
 }
 
 GST_BOILERPLATE_FULL (GstDSPAACSink, gst_dspaacsink, GstBaseSink,
                       GST_TYPE_BASE_SINK, _do_init);
+#else
+GST_BOILERPLATE (GstDSPAACSink, gst_dspaacsink, GstBaseSink, 
+                 GST_TYPE_BASE_SINK);
+#endif
 
 
-/**
+/*
  * gst_dspaacsink_dispose:
  * @object: GObject pointer to element to be deleted
  *
  * Deletes AAC sink element instance. Called automatically by
  * GLib framework when element needs to be disposed.
  */
-
 static void
 gst_dspaacsink_dispose (GObject * object)
 {
   GstDSPAACSink *dsp = (GstDSPAACSink *) object;
 
-  if(dsp->audio->clock) {
-    gst_object_unref(dsp->audio->clock);
-    dsp->audio->clock = NULL;
+  if(dsp->clock) {
+    gst_object_unref(dsp->clock);
+    dsp->clock = NULL;
   }
 
   if(dsp->audio) {
@@ -129,33 +124,37 @@ gst_dspaacsink_dispose (GObject * object)
 }
 
 
-/**
+/*
  * gst_dspaacsink_base_init:
  * @g_class: AAC sink class
  *
  * Does the basic initialization of AAC sink element class. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspaacsink_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  static const GstElementDetails details = GST_ELEMENT_DETAILS (
+    "DSP AAC Sink",
+    "Sink/Audio",
+    "AAC audio sink",
+    "Makoto Sugano <makoto.sugano@nokia.com>");
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&dspaacsink_sink_template));
-  gst_element_class_set_details (gstelement_class, &gst_dspaacsink_details);
+  gst_element_class_set_details (gstelement_class, &details);
 
 }
 
-/**
+
+/*
  * gst_dspaacsink_class_init:
  * @klass: AAC Sink class object
  *
  * Initializes AAC sink element class. This is called by GStreamer
  * framework.
  */
-
 static void
 gst_dspaacsink_class_init (GstDSPAACSinkClass * klass)
 {
@@ -192,7 +191,7 @@ gst_dspaacsink_class_init (GstDSPAACSinkClass * klass)
 }
 
 
-/**
+/*
  * gst_dspaacsink_init:
  * @dsp: DSP AAC sink object
  *
@@ -200,19 +199,20 @@ gst_dspaacsink_class_init (GstDSPAACSinkClass * klass)
  * resources. This also creates an audio stream object. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspaacsink_init (GstDSPAACSink * dspaacsink, GstDSPAACSinkClass *g_class)
 {
   dspaacsink->audio = gst_dspaudio_new();
   dspaacsink->adapter = gst_adapter_new();
-  dspaacsink->audio->clock = gst_audio_clock_new ("clock",
-                                                  (GstAudioClockGetTimeFunc) gst_dspaacsink_get_time,
-                                                  dspaacsink);
+  gst_base_sink_set_sync (GST_BASE_SINK (dspaacsink), FALSE);
+
+  dspaacsink->clock = gst_audio_clock_new ("clock",
+                                           (GstAudioClockGetTimeFunc) gst_dspaacsink_get_time,
+                                           dspaacsink);
 }
 
 
-/**
+/*
  * gst_dspaacsink_handle_sink_event:
  * @dsp: DSP AAC sink object
  *
@@ -221,7 +221,6 @@ gst_dspaacsink_init (GstDSPAACSink * dspaacsink, GstDSPAACSinkClass *g_class)
  *
  * Returns: TRUE if event was not EOS, otherwise FALSE
  */
-
 static gboolean
   gst_dspaacsink_send_remaining (GstDSPAACSink *dsp)
 {
@@ -272,14 +271,13 @@ static gboolean
 }
 
 
-/**
+/*
  * gst_dspaacsink_setcaps:
  * @sink: GstBaseSink
  * @caps: GStCaps offered
  *
  * Returns: TRUE if caps are accepted
  */
-
 static gboolean
 gst_dspaacsink_setcaps(GstBaseSink *sink, GstCaps *caps)
 {
@@ -334,7 +332,7 @@ gst_dspaacsink_setcaps(GstBaseSink *sink, GstCaps *caps)
 }
 
 
-/**
+/*
  * gst_dspaacsink_get_stream_details:
  * @dsp: DSP AAC sink object
  *
@@ -344,8 +342,6 @@ gst_dspaacsink_setcaps(GstBaseSink *sink, GstCaps *caps)
  *
  * Returns: TRUE if the info was found, otherwise FALSE
  */
-
-
 static gboolean
 gst_dspaacsink_get_stream_details(GstDSPAACSink *dsp)
 {
@@ -453,11 +449,10 @@ gst_dspaacsink_get_stream_details(GstDSPAACSink *dsp)
 }
 
 
-/**
- *
+/*
+ * gst_dspaacsink_find_sync:
  *
  */
-
 static gboolean
 gst_dspaacsink_find_sync(GstDSPAACSink *dsp)
 {
@@ -490,13 +485,12 @@ gst_dspaacsink_find_sync(GstDSPAACSink *dsp)
 }
 
 
-/**
+/*
  * gst_dspaacsink_send_discont:
  * @dsp: DSP AAC sink object
  *
  * Send a DISCONT command to DSP
  */
-
 static void
 gst_dspaacsink_send_discont(GstDSPAACSink *dsp)
 {
@@ -509,17 +503,17 @@ gst_dspaacsink_send_discont(GstDSPAACSink *dsp)
 }
 
 
-/**
- * gst_dspaacsink_render:
- * @pad:
- * @buffer:
- *
- */
 
 #define AAC_CHUNK_SIZE 500
 #define AAC_SYNC_SIZE 100000
 #define AAC_DETECT_BLOCK_SIZE 4096
 
+/*
+ * gst_dspaacsink_render:
+ * @pad:
+ * @buffer:
+ *
+ */
 static GstFlowReturn
 gst_dspaacsink_render(GstBaseSink *sink, GstBuffer * buffer)
 {
@@ -647,7 +641,6 @@ gst_dspaacsink_render(GstBaseSink *sink, GstBuffer * buffer)
       break; // Cannot continue, not enough data to find sync
     }
 
-
     data = gst_adapter_peek(dsp->adapter, AAC_CHUNK_SIZE);
 //     DBG_PRINT("AAC: Got %d bytes\n", AAC_CHUNK_SIZE);
     avail -= AAC_CHUNK_SIZE;
@@ -676,7 +669,7 @@ gst_dspaacsink_render(GstBaseSink *sink, GstBuffer * buffer)
 }
 
 
-/**
+/*
  * gst_dspaacsink_initialize:
  * @dsp: DSP AAC sink object
  *
@@ -685,7 +678,6 @@ gst_dspaacsink_render(GstBaseSink *sink, GstBuffer * buffer)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 static gboolean
 gst_dspaacsink_initialize(GstDSPAACSink *dsp)
 {
@@ -732,7 +724,7 @@ gst_dspaacsink_initialize(GstDSPAACSink *dsp)
 }
 
 
-/**
+/*
  * gst_dspaacsink_preroll:
  * @sink:
  * @buffer:
@@ -741,7 +733,6 @@ gst_dspaacsink_initialize(GstDSPAACSink *dsp)
  * a scheduler decides to give execution to this element.
  * The purpose is to check the media stream.
  */
-
 static GstFlowReturn
 gst_dspaacsink_preroll (GstBaseSink *sink, GstBuffer * buffer)
 {
@@ -777,7 +768,7 @@ gst_dspaacsink_preroll (GstBaseSink *sink, GstBuffer * buffer)
 }
 
 
-/**
+/*
  * gst_dspaacsink_set_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -786,7 +777,6 @@ gst_dspaacsink_preroll (GstBaseSink *sink, GstBuffer * buffer)
  *
  * Stores the property value into GObject
  */
-
 static void
 gst_dspaacsink_set_property(GObject * object,
                             guint prop_id,
@@ -802,7 +792,7 @@ gst_dspaacsink_set_property(GObject * object,
 }
 
 
-/**
+/*
  * gst_dspaacsink_get_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -811,7 +801,6 @@ gst_dspaacsink_set_property(GObject * object,
  *
  * Retrieves the desired parameter value from object
  */
-
 static void
 gst_dspaacsink_get_property(GObject * object,
                             guint prop_id,
@@ -826,13 +815,13 @@ gst_dspaacsink_get_property(GObject * object,
   }
 }
 
-/**
+
+/*
  * gst_dspaacsink_get_query_types:
  * @element: GstElement that received this query
  *
  * Returns: a list of supported query types
  */
-
 static const GstQueryType *
 gst_dspaacsink_get_query_types(GstElement * element)
 {
@@ -846,7 +835,7 @@ gst_dspaacsink_get_query_types(GstElement * element)
 }
 
 
-/**
+/*
  * gst_dspaacsink_query:
  * @element: GstElement that received this query
  * @type: Type of the query, e.g. TOTAL, POSITION
@@ -855,7 +844,6 @@ gst_dspaacsink_get_query_types(GstElement * element)
  *
  * Returns: TRUE, iff the query was successful.
  */
-
 static gboolean
 gst_dspaacsink_query (GstElement * element, GstQuery *query)
 {
@@ -894,7 +882,7 @@ gst_dspaacsink_query (GstElement * element, GstQuery *query)
 }
 
 
-/**
+/*
  * gst_dspaacsink_event:
  * @element: GstElement that received this event
  * @event: The GstEvent structure itself
@@ -925,12 +913,13 @@ gst_dspaacsink_query (GstElement * element, GstQuery *query)
 // }
 
 
-/**
+/*
+ * gst_dspaacsink_event2:
  *
  * This method handles events coming from basesink
  */
-
-static gboolean gst_dspaacsink_event2 (GstBaseSink *sink, GstEvent * event)
+static gboolean
+gst_dspaacsink_event2 (GstBaseSink *sink, GstEvent * event)
 {
   DBG_PRINT("^^^^^^^^^^^^^^^^^^^GOT EVENT FROM BASESINK: %d\n", GST_EVENT_TYPE (event));
   GstDSPAACSink *dsp = GST_DSPAACSINK (sink);
@@ -965,8 +954,7 @@ static gboolean gst_dspaacsink_event2 (GstBaseSink *sink, GstEvent * event)
 }
 
 
-
-/**
+/*
  * gst_dspaacsink_change_state:
  * @element: The GstElement whose state to be changed
  *
@@ -977,15 +965,12 @@ static gboolean gst_dspaacsink_event2 (GstBaseSink *sink, GstEvent * event)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 static GstStateChangeReturn
 gst_dspaacsink_change_state(GstElement * element, GstStateChange transition)
 {
   GstDSPAACSink *dspaacsink = GST_DSPAACSINK (element);
   GstStateChangeReturn ret;
 
-  gst_dspaudio_clock_change_state(dspaacsink->audio, transition);
-
   switch (transition) {
 
     case GST_STATE_CHANGE_NULL_TO_READY:
@@ -1039,6 +1024,7 @@ gst_dspaacsink_change_state(GstElement * element, GstStateChange transition)
       break;
   }
 
+  gst_dspaudio_clock_change_state(dspaacsink->audio, transition);
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
@@ -1082,25 +1068,23 @@ gst_dspaacsink_change_state(GstElement * element, GstStateChange transition)
 }
 
 
-/**
- *
+/*
+ * gst_dspaacsink_provide_clock:
  *
  */
-
 static GstClock *
 gst_dspaacsink_provide_clock (GstElement * elem)
 {
   DBG_PRINT("gst_dspaacsink_provide_clock\n");
   GstDSPAACSink *dsp = GST_DSPAACSINK (elem);
-  return GST_CLOCK (gst_object_ref (GST_OBJECT (dsp->audio->clock)));
+  return GST_CLOCK (gst_object_ref (GST_OBJECT (dsp->clock)));
 }
 
 
-/**
- *
+/*
+ * gst_dspaacsink_get_time:
  *
  */
-
 static GstClockTime
 gst_dspaacsink_get_time (GstClock * clock, gpointer sink)
 {
@@ -1109,7 +1093,7 @@ gst_dspaacsink_get_time (GstClock * clock, gpointer sink)
 }
 
 
-/**
+/*
  * plugin_init:
  * @plugin: GstPlugin to be initialized
  *
@@ -1117,7 +1101,6 @@ gst_dspaacsink_get_time (GstClock * clock, gpointer sink)
  *
  * Returns: Boolean value, TRUE if initialization was successful
  */
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -1125,12 +1108,6 @@ plugin_init (GstPlugin * plugin)
       GST_TYPE_DSPAACSINK);
 }
 
-
-/**
- * This is the "boilerplate" of the plugin
- *
- */
-
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
                    GST_VERSION_MINOR,
                    "dspaac",
@@ -1139,4 +1116,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
                    VERSION,
                    "LGPL",
                    "dspaudio",
-                   "Nokia Corporation");
+                   "http://www.nokia.com/");
