@@ -40,7 +40,7 @@
 
 #define AVG_LEN_CALC_THRESHOLD (5 * 60 * AMR_FPS)
 
-const short int FrameLength[2][16] =
+static const short int FrameLength[2][16] =
 {
   {13, 14, 16, 18, 20, 21, 27, 32,  6, 1, 1, 1, 1, 1, 1, 1},    // Narrow-band
   {18, 24, 33, 37, 41, 47, 51, 59, 61, 6, 6, 1, 1, 1, 1, 1}     // Wideband
@@ -48,18 +48,12 @@ const short int FrameLength[2][16] =
 
 static guint gst_dspamrsink_signals[DSPAMR_SIGNAL_LAST] = { 0 };
 
-const gchar *devname[2] =
+static const gchar *devname[2] =
 {
   "/dev/dsptask/amrnb",
   "/dev/dsptask/amrwb"
 };
 
-GstElementDetails gst_dspamrsink_details =
-GST_ELEMENT_DETAILS ("DSP AMR Sink",
-                     "Sink/Audio",
-                     "AMR NB/WB audio sink",
-                     "Makoto Sugano <makoto.sugano@nokia.com>");
-
 
 static GstStaticPadTemplate dspamrsink_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -95,31 +89,35 @@ static GstStateChangeReturn gst_dspamrsink_async_play(GstBaseSink *sink);
 // static void gst_dspamrsink_calculate_length(GstDSPAMRSink *dsp);
 
 
-void _do_init()
+#ifdef DEBUG
+static void _do_init()
 {
   DBG_PRINT("AMR DO_INIT\n");
 }
 
 GST_BOILERPLATE_FULL (GstDSPAMRSink, gst_dspamrsink, GstBaseSink,
                       GST_TYPE_BASE_SINK, _do_init);
+#else
+GST_BOILERPLATE (GstDSPAMRSink, gst_dspamrsink, GstBaseSink,
+                 GST_TYPE_BASE_SINK);
+#endif
 
 
-/**
+/*
  * gst_dspamrsink_dispose:
  * @object: GObject pointer to element to be deleted
  *
  * Deletes AMR sink element instance. Called automatically by
  * GLib framework when element needs to be disposed.
  */
-
 static void
 gst_dspamrsink_dispose (GObject * object)
 {
   GstDSPAMRSink *dsp = (GstDSPAMRSink *) object;
 
-  if(dsp->audio->clock) {
-    gst_object_unref(dsp->audio->clock);
-    dsp->audio->clock = NULL;
+  if(dsp->clock) {
+    gst_object_unref(dsp->clock);
+    dsp->clock = NULL;
   }
 
   if(dsp->audio) {
@@ -132,33 +130,37 @@ gst_dspamrsink_dispose (GObject * object)
 }
 
 
-/**
+/*
  * gst_dspamrsink_base_init:
  * @g_class: AMR sink class
  *
  * Does the basic initialization of AMR sink element class. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspamrsink_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  static const GstElementDetails details = GST_ELEMENT_DETAILS (
+    "DSP AMR Sink",
+    "Sink/Audio",
+    "AMR NB/WB audio sink",
+    "Makoto Sugano <makoto.sugano@nokia.com>");
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&dspamrsink_sink_template));
-  gst_element_class_set_details (gstelement_class, &gst_dspamrsink_details);
+  gst_element_class_set_details (gstelement_class, &details);
 
 }
 
-/**
+
+/*
  * gst_dspamrsink_class_init:
  * @klass: AMR Sink class object
  *
  * Initializes AMR sink element class. This is called by GStreamer
  * framework.
  */
-
 static void
 gst_dspamrsink_class_init (GstDSPAMRSinkClass * klass)
 {
@@ -196,7 +198,7 @@ gst_dspamrsink_class_init (GstDSPAMRSinkClass * klass)
 }
 
 
-/**
+/*
  * gst_dspamrsink_init:
  * @dsp: DSP AMR sink object
  *
@@ -204,7 +206,6 @@ gst_dspamrsink_class_init (GstDSPAMRSinkClass * klass)
  * resources. This also creates an audio stream object. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspamrsink_init (GstDSPAMRSink * dspamrsink, GstDSPAMRSinkClass *g_class)
 {
@@ -213,21 +214,21 @@ gst_dspamrsink_init (GstDSPAMRSink * dspamrsink, GstDSPAMRSinkClass *g_class)
   dspamrsink->sinkpad = bsink->sinkpad;
   dspamrsink->audio = gst_dspaudio_new();
   dspamrsink->adapter = gst_adapter_new();
+  gst_base_sink_set_sync (GST_BASE_SINK (dspamrsink), FALSE);
 
-  dspamrsink->audio->clock = gst_audio_clock_new ("clock",
-                                                  (GstAudioClockGetTimeFunc) gst_dspamrsink_get_time,
-                                                  dspamrsink);
+  dspamrsink->clock = gst_audio_clock_new ("clock",
+                                           (GstAudioClockGetTimeFunc) gst_dspamrsink_get_time,
+                                           dspamrsink);
 }
 
 
-/**
+/*
  * gst_dspamrsink_setcaps:
  * @basesink: GstBaseSink
  * @caps: GstCaps that were offered by the connecting element
  *
  * Returns: TRUE if caps can be accepted, otherwise FALSE
  */
-
 static gboolean
 gst_dspamrsink_setcaps(GstBaseSink * basesink, GstCaps * caps)
 {
@@ -255,12 +256,12 @@ gst_dspamrsink_setcaps(GstBaseSink * basesink, GstCaps * caps)
 }
 
 
-/**
+/*
  * gst_dspamrsink_send_discont:
  *
  */
-
-static void gst_dspamrsink_send_discont(GstDSPAMRSink *dsp)
+static void
+gst_dspamrsink_send_discont(GstDSPAMRSink *dsp)
 {
   if(gst_dspaudio_discont(dsp->audio)) {
     DBG_PRINT("DISCONT written... reading reply\n");
@@ -271,7 +272,8 @@ static void gst_dspamrsink_send_discont(GstDSPAMRSink *dsp)
   }
 }
 
-/**
+
+/*
  * gst_dspamrsink_fill_input_buffer;
  * @dsp: GstDSPAMRSink to be handled
  *
@@ -280,8 +282,7 @@ static void gst_dspamrsink_send_discont(GstDSPAMRSink *dsp)
  *
  * Returns: TRUE if buffer was filled, otherwise FALSE
  */
-
-gboolean
+static gboolean
 gst_dspamrsink_fill_input_buffer(GstDSPAMRSink *dsp)
 {
   guint frame_index, frame_length;
@@ -343,12 +344,12 @@ gst_dspamrsink_fill_input_buffer(GstDSPAMRSink *dsp)
 }
 
 
-/**
- *
+/*
+ * gst_dspamrsink_perform_seek:
  *
  */
-
-gboolean gst_dspamrsink_perform_seek(GstDSPAMRSink *dsp, gint64 seek_pos)
+static gboolean
+gst_dspamrsink_perform_seek(GstDSPAMRSink *dsp, gint64 seek_pos)
 {
   guint64 pos_in_frames = seek_pos / AMR_TPF;
   gint64 pos = -1;
@@ -383,13 +384,12 @@ gboolean gst_dspamrsink_perform_seek(GstDSPAMRSink *dsp, gint64 seek_pos)
 }
 
 
-/**
+/*
  * gst_dspamrsink_render:
  * @sink:
  * @buffer:
  *
  */
-
 static GstFlowReturn
 gst_dspamrsink_render(GstBaseSink *sink, GstBuffer * buffer)
 {
@@ -466,7 +466,7 @@ gst_dspamrsink_render(GstBaseSink *sink, GstBuffer * buffer)
 }
 
 
-/**
+/*
  * gst_dspamrsink_initialize:
  * @dsp: DSP AMR sink object
  *
@@ -475,7 +475,6 @@ gst_dspamrsink_render(GstBaseSink *sink, GstBuffer * buffer)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 static gboolean
 gst_dspamrsink_initialize(GstDSPAMRSink *dsp)
 {
@@ -510,7 +509,7 @@ gst_dspamrsink_initialize(GstDSPAMRSink *dsp)
 }
 
 
-/**
+/*
  * gst_dspamrsink_set_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -519,7 +518,6 @@ gst_dspamrsink_initialize(GstDSPAMRSink *dsp)
  *
  * Stores the property value into GObject
  */
-
 static void
 gst_dspamrsink_set_property (GObject * object,
                              guint prop_id,
@@ -534,7 +532,7 @@ gst_dspamrsink_set_property (GObject * object,
 }
 
 
-/**
+/*
  * gst_dspamrsink_get_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -543,7 +541,6 @@ gst_dspamrsink_set_property (GObject * object,
  *
  * Retrieves the desired parameter value from object
  */
-
 static void
 gst_dspamrsink_get_property (GObject * object,
                              guint prop_id,
@@ -558,13 +555,12 @@ gst_dspamrsink_get_property (GObject * object,
 }
 
 
-/**
+/*
  * gst_dspamrsink_get_query_types:
  * @element: GstElement that received this query
  *
  * Returns: a list of supported query types
  */
-
 static const GstQueryType *
 gst_dspamrsink_get_query_types (GstElement * element)
 {
@@ -577,15 +573,13 @@ gst_dspamrsink_get_query_types (GstElement * element)
 }
 
 
-/**
+/*
  * gst_dspamrsink_query:
  * @element: GstElement that received this query
  * @query: GstQuery object
  *
  * Returns: TRUE, iff the query was successful.
  */
-
-
 static gboolean
 gst_dspamrsink_query (GstElement * element, GstQuery *query)
 {
@@ -629,14 +623,13 @@ gst_dspamrsink_query (GstElement * element, GstQuery *query)
 }
 
 
-/**
+/*
  * gst_dspamrsink_event:
  * @element: GstElement that received this event
  * @event: The GstEvent structure itself
  *
  * Returns: TRUE, iff the event was successful
  */
-
 static gboolean
 gst_dspamrsink_event (GstElement * element, GstEvent * event)
 {
@@ -679,11 +672,11 @@ gst_dspamrsink_event (GstElement * element, GstEvent * event)
 }
 
 
-/**
+/*
+ * gst_dspamrsink_event2:
  *
  * This method handles events coming from basesink
  */
-
 static gboolean
 gst_dspamrsink_event2 (GstBaseSink *sink, GstEvent * event)
 {
@@ -716,7 +709,7 @@ gst_dspamrsink_event2 (GstBaseSink *sink, GstEvent * event)
 }
 
 
-/**
+/*
  * gst_dspamrsink_check_stream:
  * @dsp: DSP AMR sink object
  *
@@ -725,8 +718,7 @@ gst_dspamrsink_event2 (GstBaseSink *sink, GstEvent * event)
  *
  * Returns: TRUE if the content was recognized, otherwise FALSE
  */
-
-gboolean
+static gboolean
 gst_dspamrsink_check_stream(GstDSPAMRSink *dsp, const guint8 *data)
 {
   gboolean retval = FALSE;
@@ -751,12 +743,12 @@ gst_dspamrsink_check_stream(GstDSPAMRSink *dsp, const guint8 *data)
 }
 
 
-/**
- *
+/*
+ * gst_dspamrsink_open:
  *
  */
-
-static gboolean gst_dspamrsink_open(GstDSPAMRSink *dsp, guint bandindex)
+static gboolean
+gst_dspamrsink_open(GstDSPAMRSink *dsp, guint bandindex)
 {
   if(!gst_dspaudio_open(dsp->audio, devname[dsp->bandindex])) {
     GST_ELEMENT_ERROR (dsp, RESOURCE,
@@ -772,11 +764,10 @@ static gboolean gst_dspamrsink_open(GstDSPAMRSink *dsp, guint bandindex)
 }
 
 
-/**
- *
+/*
+ * gst_dspamrsink_paused_to_playing:
  *
  */
-
 static gboolean
 gst_dspamrsink_paused_to_playing(GstDSPAMRSink *dsp)
 {
@@ -790,7 +781,6 @@ gst_dspamrsink_paused_to_playing(GstDSPAMRSink *dsp)
   g_signal_emit (G_OBJECT (dsp), gst_dspamrsink_signals[DSPAMR_SIGNAL_STREAMID],
                  0, dsp->audio->codec.stream_id);
 
-
   // AMR-specific initialization part
   if(!gst_dspamrsink_initialize(dsp)) {
     GST_ELEMENT_ERROR (dsp, RESOURCE, SETTINGS, (NULL),
@@ -808,14 +798,13 @@ gst_dspamrsink_paused_to_playing(GstDSPAMRSink *dsp)
 }
 
 
-/**
+/*
  * gst_dspamrsink_query_length:
  * @dsp: GstDSPAMRSink object
  *
  * Retrieves the length of stream, in bytes. Stores the value
  * in dsp->total_length_bytes variable
  */
-
 static void
 gst_dspamrsink_query_length(GstDSPAMRSink *dsp)
 {
@@ -835,11 +824,10 @@ gst_dspamrsink_query_length(GstDSPAMRSink *dsp)
 }
 
 
-/**
- *
+/*
+ * gst_dspamrsink_preroll:
  *
  */
-
 static GstFlowReturn
 gst_dspamrsink_preroll(GstBaseSink *sink, GstBuffer * buffer)
 {
@@ -873,14 +861,13 @@ gst_dspamrsink_preroll(GstBaseSink *sink, GstBuffer * buffer)
 }
 
 
-/**
+/*
  * gst_dspamrsink_change_state:
  * @element: The GstElement whose state to be changed
  *
  * Returns: GST_STATE_CHANGE_SUCCESS if operation was successful.
  *          Otherwise GST_STATE_CHANGE_FAILURE will be returned.
  */
-
 static GstStateChangeReturn
 gst_dspamrsink_change_state(GstElement * element, GstStateChange transition)
 {
@@ -889,8 +876,6 @@ gst_dspamrsink_change_state(GstElement * element, GstStateChange transition)
 
   DBG_PRINT("AMR CHANGE_STATE CALLED\n");
 
-  gst_dspaudio_clock_change_state(dspamrsink->audio, transition);
-
   switch (transition) {
 
     case GST_STATE_CHANGE_NULL_TO_READY:
@@ -933,6 +918,7 @@ gst_dspamrsink_change_state(GstElement * element, GstStateChange transition)
       break;
   }
 
+  gst_dspaudio_clock_change_state(dspamrsink->audio, transition);
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
@@ -975,11 +961,10 @@ gst_dspamrsink_change_state(GstElement * element, GstStateChange transition)
 }
 
 
-/**
- *
+/*
+ * gst_dspamrsink_async_play:
  *
  */
-
 static GstStateChangeReturn
 gst_dspamrsink_async_play (GstBaseSink *sink)
 {
@@ -996,25 +981,23 @@ gst_dspamrsink_async_play (GstBaseSink *sink)
 }
 
 
-/**
- *
+/*
+ * gst_dspamrsink_provide_clock:
  *
  */
-
 static GstClock *
 gst_dspamrsink_provide_clock (GstElement * elem)
 {
   DBG_PRINT("gst_dspamrsink_provide_clock\n");
   GstDSPAMRSink *dsp = GST_DSPAMRSINK (elem);
-  return GST_CLOCK (gst_object_ref (GST_OBJECT (dsp->audio->clock)));
+  return GST_CLOCK (gst_object_ref (GST_OBJECT (dsp->clock)));
 }
 
 
-/**
- *
+/*
+ * gst_dspamrsink_get_time:
  *
  */
-
 static GstClockTime
 gst_dspamrsink_get_time (GstClock * clock, gpointer sink)
 {
@@ -1023,41 +1006,6 @@ gst_dspamrsink_get_time (GstClock * clock, gpointer sink)
 }
 
 
-/**
- * plugin_init:
- * @plugin: GstPlugin to be initialized
- *
- * This initializes the plugin when it is loaded by GStreamer
- *
- * Returns: Boolean value, TRUE if initialization was successful
- */
-
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "dspamrsink", GST_RANK_PRIMARY,
-      GST_TYPE_DSPAMRSINK);
-}
-
-
-/**
- * This is the "boilerplate" of the plugin
- *
- */
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-                   GST_VERSION_MINOR,
-                   "dspamr",
-                   "DSP AMR NB/WB audio sink",
-                   plugin_init,
-                   VERSION,
-                   "LGPL",
-                   "dspaudio",
-                   "Nokia Corporation");
-
-
-
-
 #define AMR_LEN_CALC_BLOCKSIZE (4*1024)
 
 /* For now this can wait until we upgrade the plugin to work
@@ -1151,3 +1099,29 @@ gst_dspamrsink_calculate_length(GstDSPAMRSink *dsp)
 }
 */
 
+
+/*
+ * plugin_init:
+ * @plugin: GstPlugin to be initialized
+ *
+ * This initializes the plugin when it is loaded by GStreamer
+ *
+ * Returns: Boolean value, TRUE if initialization was successful
+ */
+static gboolean
+plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "dspamrsink", GST_RANK_PRIMARY,
+      GST_TYPE_DSPAMRSINK);
+}
+
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+                   GST_VERSION_MINOR,
+                   "dspamr",
+                   "DSP AMR NB/WB audio sink",
+                   plugin_init,
+                   VERSION,
+                   "LGPL",
+                   "dspaudio",
+                   "http://www.nokia.com/");
+
