@@ -33,7 +33,6 @@
 #include <sys/mman.h>
 #include <sys/poll.h>
 
-
 #include <gst/gst.h>
 
 #include "dspaudio.h"
@@ -53,13 +52,59 @@
 #define RESOURCE_INTERFACE          "com.nokia.osso_resource_manager"
 #define RESOURCE_TIMEOUT            200
 
+
+static gboolean 
+gst_dspaudio_open_node(DSPNodeInfo *info, const gchar *devname);
+static gboolean
+gst_dspaudio_close_node(DSPNodeInfo *info);
+static gboolean
+gst_dspaudio_check_read(int dev_fd);
+static gboolean
+gst_dspaudio_set_volume(GstDSPAudio *dsp);
+static gboolean
+gst_dspaudio_set_panning(GstDSPAudio *dsp);
+static gboolean
+gst_dspaudio_set_mute(GstDSPAudio *dsp);
+static gboolean
+gst_dspaudio_reset_node(GstDSPAudio *dsp);
+static gboolean
+gst_dspaudio_mic_ctl(GstDSPAudio *dsp, gboolean mic_enable);
+static void
+gst_dspaudio_notify_render(GstDSPAudio *dsp);
+static gboolean
+gst_dspaudio_clock_interrupt(GstDSPAudio *dsp);
+static GstClockTime
+gst_dspaudio_get_dsp_time(GstDSPAudio *dsp);
+static GstBuffer *
+gst_dspaudio_default_voice_frame_handler (short int *mmap_ptr, guint fsize);
+static GstBuffer *
+gst_dspaudio_default_sid_frame_handler (short int *mmap_ptr, guint fsize);
+
+
+GType
+gst_dspaudio_dtx_mode_get_type (void)
+{
+  static GType dspaudio_dtx_mode_type = 0;
+  static GEnumValue dspaudio_dtx_modes[] = {
+    {GST_DSPAUDIO_DTX_MODE_OFF, "Don't use DTX", "off"},
+    {GST_DSPAUDIO_DTX_MODE_ON, "Use DTX", "on"},
+    {0, NULL, NULL},
+  };
+
+  if (G_UNLIKELY(!dspaudio_dtx_mode_type)) {
+    dspaudio_dtx_mode_type = g_enum_register_static ("GstDSPAudioDTXMode",
+        dspaudio_dtx_modes);
+  }
+  return dspaudio_dtx_mode_type;
+}
+
+
 /**
  * gst_dspaudio_reset:
  * @dsp: DSP audio object
  *
  * Method for resetting the audio object
  */
-
 void
 gst_dspaudio_reset(GstDSPAudio * dsp)
 {
@@ -80,8 +125,11 @@ gst_dspaudio_reset(GstDSPAudio * dsp)
   //dsp->mic_enabled = FALSE;
   dsp->read_sent = FALSE;
   dsp->interrupted = FALSE;
-
+  dsp->dtmf_playing = FALSE;
   FD_ZERO(&dsp->read_fdset);
+
+  dsp->handle_voice_frame = gst_dspaudio_default_voice_frame_handler;
+  dsp->handle_sid_frame = gst_dspaudio_default_sid_frame_handler;
 }
 
 
@@ -94,11 +142,10 @@ gst_dspaudio_reset(GstDSPAudio * dsp)
  *
  * Returns: New #GstDSPAudio instance
  */
-
 GstDSPAudio *
 gst_dspaudio_new()
 {
-  GstDSPAudio *dsp = g_new(GstDSPAudio, 1);
+  GstDSPAudio *dsp = g_new0(GstDSPAudio, 1);
   dsp->volume = DSP_VOLUME_DEFAULT;
   dsp->panning = DSP_PANNING_DEFAULT;
   dsp->priority = DSP_PRIORITY_DEFAULT;
@@ -127,7 +174,6 @@ gst_dspaudio_new()
  *
  * Delete given audio object instance
  */
-
 void
 gst_dspaudio_destroy(GstDSPAudio * dsp)
 {
@@ -149,7 +195,7 @@ gst_dspaudio_destroy(GstDSPAudio * dsp)
 }
 
 
-/**
+/*
  * gst_dspaudio_open_node:
  * @dsp: DSP node info object
  * @devname: device name to be opened & initialized
@@ -158,8 +204,8 @@ gst_dspaudio_destroy(GstDSPAudio * dsp)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
-gboolean gst_dspaudio_open_node(DSPNodeInfo *info, const gchar *devname)
+static gboolean
+gst_dspaudio_open_node(DSPNodeInfo *info, const gchar *devname)
 {
   AUDIO_INIT_STATUS init_status;
   AUDIO_STATUS_INFO statInfo;
@@ -174,10 +220,10 @@ gboolean gst_dspaudio_open_node(DSPNodeInfo *info, const gchar *devname)
     return FALSE;
   }
 
-//   if(ioctl(info->fd, OMAP_DSP_TASK_IOCTL_LOCK) < 0) {
-//     DBG_PRINT("Locking of %s failed\n", devname);
-//     return FALSE;
-//   }
+  //if(ioctl(info->fd, OMAP_DSP_TASK_IOCTL_LOCK) < 0) {
+  //  DBG_PRINT("Locking of %s failed\n", devname);
+  //  return FALSE;
+  //}
 
   // First we try to read all "old stuff"
   gst_dspaudio_flush(info);
@@ -234,7 +280,7 @@ gboolean gst_dspaudio_open_node(DSPNodeInfo *info, const gchar *devname)
 }
 
 
-/**
+/*
  * gst_dspaudio_close_node:
  * @dsp: DSP node info object
  *
@@ -242,8 +288,8 @@ gboolean gst_dspaudio_open_node(DSPNodeInfo *info, const gchar *devname)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
-gboolean gst_dspaudio_close_node(DSPNodeInfo *info)
+static gboolean
+gst_dspaudio_close_node(DSPNodeInfo *info)
 {
   DSP_CMD_STATUS cmd;
   DBG_PRINT("gst_dspaudio_close_node\n");
@@ -256,7 +302,7 @@ gboolean gst_dspaudio_close_node(DSPNodeInfo *info)
     gst_dspaudio_flush(info);
 
     cmd.dsp_cmd = DSP_CMD_CLOSE;
-    if(write(info->fd, &cmd, sizeof(short int)) < 0) {
+    if(write(info->fd, &cmd, sizeof(short int)) > 0) {
       read(info->fd, &cmd, sizeof(DSP_CMD_STATUS));
     }
 
@@ -287,8 +333,8 @@ gboolean gst_dspaudio_close_node(DSPNodeInfo *info)
  *
  * Reads all unread data from node
  */
-
-void gst_dspaudio_flush(DSPNodeInfo *info)
+void
+gst_dspaudio_flush(DSPNodeInfo *info)
 {
   short int tmp;
 
@@ -305,7 +351,6 @@ void gst_dspaudio_flush(DSPNodeInfo *info)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_open(GstDSPAudio *dsp, const gchar *devname)
 {
@@ -336,6 +381,7 @@ gst_dspaudio_open(GstDSPAudio *dsp, const gchar *devname)
   return TRUE;
 }
 
+
 /**
  * gst_dspaudio_close:
  * @dsp: DSP audio object
@@ -344,14 +390,15 @@ gst_dspaudio_open(GstDSPAudio *dsp, const gchar *devname)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
-gboolean gst_dspaudio_close(GstDSPAudio *dsp)
+gboolean
+gst_dspaudio_close(GstDSPAudio *dsp)
 {
   gboolean ret = gst_dspaudio_close_node(&dsp->codec);
   dsp->mode = DSP_MODE_UNINITIALIZED;
   return ret;
 }
 
+
 /**
  * gst_dspaudio_setparams:
  * @dsp: DSP audio object
@@ -361,7 +408,6 @@ gboolean gst_dspaudio_close(GstDSPAudio *dsp)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_setparams(GstDSPAudio *dsp, char *data, guint datasize)
 {
@@ -399,7 +445,7 @@ gst_dspaudio_setparams(GstDSPAudio *dsp, char *data, guint datasize)
 }
 
 
-/**
+/*
  * gst_dspaudio_check_read:
  * @dsp: DSP audio object
  *
@@ -407,8 +453,7 @@ gst_dspaudio_setparams(GstDSPAudio *dsp, char *data, guint datasize)
  *
  * Returns: TRUE if data is available, otherwise FALSE
  */
-
-gboolean
+static gboolean
 gst_dspaudio_check_read(int dev_fd)
 {
   struct pollfd pfd;
@@ -420,6 +465,7 @@ gst_dspaudio_check_read(int dev_fd)
   else return FALSE;
 }
 
+
 /**
  * gst_dspaudio_read_cmd:
  * @dsp: DSP audio object
@@ -429,7 +475,6 @@ gst_dspaudio_check_read(int dev_fd)
  *
  * Returns: DSPWaitStatus
  */
-
 DSPWaitStatus
 gst_dspaudio_read_cmd(GstDSPAudio *dsp, gint timeout_millis)
 {
@@ -472,7 +517,7 @@ gst_dspaudio_read_cmd(GstDSPAudio *dsp, gint timeout_millis)
       dsp->error_cmd = DSP_CMD_NONE;
       goto read_cmd_error;
     }
-//     DBG_PRINT("READ_CMD GOT REPLY: %d, status: %d\n", data[0], data[1]);
+    //DBG_PRINT("READ_CMD GOT REPLY: %d, status: %d\n", data[0], data[1]);
 
     dsp->error_cmd = data[0];
 
@@ -515,7 +560,7 @@ gst_dspaudio_read_cmd(GstDSPAudio *dsp, gint timeout_millis)
       dsp->rw_pending++;
       continue;
     }
-    if(data[0] == DSP_CMD_DATA_READ) {
+    else if(data[0] == DSP_CMD_DATA_READ) {
       if(read(dsp->codec.fd, &(dsp->readinfo),
          sizeof(READ_STATUS) - 2*sizeof(unsigned short int)) < 0)
       {
@@ -548,7 +593,6 @@ read_cmd_error:
  *
  * Returns: DSPWaitStatus
  */
-
 DSPWaitStatus
 gst_dspaudio_wait_buffer(GstDSPAudio *dsp)
 {
@@ -565,10 +609,10 @@ gst_dspaudio_wait_buffer(GstDSPAudio *dsp)
 
   dsp->error_status = dsp->error_cmd = 0;
 
-//   if(dsp->rw_pending) {
-//     dsp->rw_pending = 0;
-//     return TRUE;
-//   }
+  //if(dsp->rw_pending) {
+  //  dsp->rw_pending = 0;
+  //  return TRUE;
+  //}
 
   while(TRUE) {
 
@@ -599,7 +643,7 @@ gst_dspaudio_wait_buffer(GstDSPAudio *dsp)
         return DSP_WAIT_INTERRUPT;
       }
       else if(tmp == PIPE_CMD_NEED_LOCK) {
-//         DBG_PRINT("PIPE_CMD_NEED_LOCK\n");
+        //DBG_PRINT("PIPE_CMD_NEED_LOCK\n");
         return DSP_WAIT_RETRY;
       }
     }
@@ -654,7 +698,7 @@ gst_dspaudio_wait_buffer(GstDSPAudio *dsp)
         }
         dsp->rw_pending++;
       }
-      if(incoming.dsp_cmd == DSP_CMD_DATA_READ) {
+      else if(incoming.dsp_cmd == DSP_CMD_DATA_READ) {
         if(read(dsp->codec.fd, &(dsp->readinfo),
            sizeof(READ_STATUS) - 2*sizeof(unsigned short int)) < 0)
         {
@@ -695,7 +739,6 @@ read_buffer_error:
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_play(GstDSPAudio *dsp)
 {
@@ -731,7 +774,6 @@ gst_dspaudio_play(GstDSPAudio *dsp)
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_pause(GstDSPAudio *dsp)
 {
@@ -770,7 +812,6 @@ gst_dspaudio_pause(GstDSPAudio *dsp)
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_stop(GstDSPAudio *dsp)
 {
@@ -811,7 +852,6 @@ gst_dspaudio_stop(GstDSPAudio *dsp)
  *
  * Waits until EOS is received from DSP
  */
-
 void
 gst_dspaudio_wait_eos(GstDSPAudio *dsp)
 {
@@ -838,7 +878,6 @@ gst_dspaudio_wait_eos(GstDSPAudio *dsp)
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_discont(GstDSPAudio *dsp)
 {
@@ -864,7 +903,7 @@ gst_dspaudio_discont(GstDSPAudio *dsp)
 }
 
 
-/**
+/*
  * gst_dspaudio_set_volume:
  * @dsp: DSP audio object
  *
@@ -874,8 +913,7 @@ gst_dspaudio_discont(GstDSPAudio *dsp)
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
-gboolean
+static gboolean
 gst_dspaudio_set_volume(GstDSPAudio *dsp)
 {
   VOLUME_DATA volume_data;
@@ -910,7 +948,7 @@ gst_dspaudio_set_volume(GstDSPAudio *dsp)
 }
 
 
-/**
+/*
  * gst_dspaudio_set_panning:
  * @dsp: DSP audio object
  *
@@ -920,8 +958,7 @@ gst_dspaudio_set_volume(GstDSPAudio *dsp)
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
-gboolean
+static gboolean
 gst_dspaudio_set_panning(GstDSPAudio *dsp)
 {
   PANNING_DATA panning_data;
@@ -959,7 +996,7 @@ gst_dspaudio_set_panning(GstDSPAudio *dsp)
 }
 
 
-/**
+/*
  * gst_dspaudio_set_mute:
  * @dsp: DSP audio object
  *
@@ -968,8 +1005,7 @@ gst_dspaudio_set_panning(GstDSPAudio *dsp)
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
-gboolean
+static gboolean
 gst_dspaudio_set_mute(GstDSPAudio *dsp)
 {
   DSP_CMD_STATUS status;
@@ -997,7 +1033,7 @@ gst_dspaudio_set_mute(GstDSPAudio *dsp)
 }
 
 
-/**
+/*
  * gst_dspaudio_reset_node:
  * @dsp: DSP audio object
  *
@@ -1005,8 +1041,7 @@ gst_dspaudio_set_mute(GstDSPAudio *dsp)
  *
  * Returns: TRUE if the operation was successful, otherwise FALSE
  */
-
-gboolean
+static gboolean
 gst_dspaudio_reset_node(GstDSPAudio *dsp)
 {
   DBG_PRINT("RESET NODE\n");
@@ -1022,7 +1057,7 @@ gst_dspaudio_reset_node(GstDSPAudio *dsp)
 }
 
 
-/**
+/*
  * gst_dspaudio_mic_ctl:
  * @dsp: DSP audio object
  * @mic_enable: If TRUE, microphone input is turned on
@@ -1031,8 +1066,7 @@ gst_dspaudio_reset_node(GstDSPAudio *dsp)
  *
  * Returns: TRUE of operation was successful
  */
-
-gboolean
+static gboolean
 gst_dspaudio_mic_ctl(GstDSPAudio *dsp, gboolean mic_enable)
 {
   DBusConnection *connection;
@@ -1087,7 +1121,6 @@ gst_dspaudio_mic_ctl(GstDSPAudio *dsp, gboolean mic_enable)
  * Sets the volume, panning and mute values to DSP if they
  * are changed since last time.
  */
-
 void
 gst_dspaudio_update_dsp_settings(GstDSPAudio *dsp)
 {
@@ -1119,14 +1152,14 @@ gst_dspaudio_update_dsp_settings(GstDSPAudio *dsp)
 }
 
 
-/**
- *
+/*
+ * gst_dspaudio_notify_render:
  *
  */
-
-void gst_dspaudio_notify_render(GstDSPAudio *dsp)
+static void
+gst_dspaudio_notify_render(GstDSPAudio *dsp)
 {
-//   DBG_PRINT("gst_dspaudio_notify_render: dsp->mode = %d\n", dsp->mode);
+  //DBG_PRINT("gst_dspaudio_notify_render: dsp->mode = %d\n", dsp->mode);
   if(dsp->mode == DSP_MODE_PLAYING) {
     char tmp = PIPE_CMD_PROP_UPDATE;
     DBG_PRINT("WRITING TO PIPE\n");
@@ -1135,14 +1168,15 @@ void gst_dspaudio_notify_render(GstDSPAudio *dsp)
   }
 }
 
+
 /**
- *
+ * gst_dspaudio_interrupt_render:
  *
  */
-
-void gst_dspaudio_interrupt_render(GstDSPAudio *dsp)
+void
+gst_dspaudio_interrupt_render(GstDSPAudio *dsp)
 {
-//   DBG_PRINT("gst_dspaudio_notify_render: dsp->mode = %d\n", dsp->mode);
+  //DBG_PRINT("gst_dspaudio_notify_render: dsp->mode = %d\n", dsp->mode);
   if(dsp->mode == DSP_MODE_PLAYING || dsp->mode == DSP_MODE_EOS) {
     char tmp = PIPE_CMD_INTERRUPT;
     DBG_PRINT("WRITING INTERRUPT CMD TO PIPE\n");
@@ -1152,12 +1186,13 @@ void gst_dspaudio_interrupt_render(GstDSPAudio *dsp)
   }
 }
 
+
 /**
- *
+ * gst_dspaudio_remove_interrupt:
  *
  */
-
-void gst_dspaudio_remove_interrupt(GstDSPAudio *dsp)
+void
+gst_dspaudio_remove_interrupt(GstDSPAudio *dsp)
 {
   char tmp;
 
@@ -1169,6 +1204,7 @@ void gst_dspaudio_remove_interrupt(GstDSPAudio *dsp)
   }
 }
 
+
 /**
  * gst_dspaudio_set_property:
  * @dsp: DSP audio object
@@ -1179,7 +1215,6 @@ void gst_dspaudio_remove_interrupt(GstDSPAudio *dsp)
  *
  * Returns: TRUE if property was known, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_set_property (GstDSPAudio *dsp, guint prop_id, const GValue *value)
 {
@@ -1244,7 +1279,6 @@ gst_dspaudio_set_property (GstDSPAudio *dsp, guint prop_id, const GValue *value)
  *
  * Returns: TRUE if property was known, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_get_property (GstDSPAudio *dsp, guint prop_id, GValue *value)
 {
@@ -1282,7 +1316,6 @@ gst_dspaudio_get_property (GstDSPAudio *dsp, guint prop_id, GValue *value)
  *
  * Adds default parameters that are common to all audio plugins to object
  */
-
 void
 gst_dspaudio_install_properties (GObjectClass *klass)
 {
@@ -1329,7 +1362,6 @@ gst_dspaudio_install_properties (GObjectClass *klass)
  *
  * Returns: TRUE if conversion was successful, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_map_samplerate(gint rate, gint *dest_rate)
 {
@@ -1363,7 +1395,6 @@ gst_dspaudio_map_samplerate(gint rate, gint *dest_rate)
  *
  * Returns: TRUE if query succeeded, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_query_peer_position(GstPad *sinkpad, gint64 *value)
 {
@@ -1387,7 +1418,6 @@ gst_dspaudio_query_peer_position(GstPad *sinkpad, gint64 *value)
  *
  * Returns: TRUE if query succeeded, otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_query_peer_duration(GstPad *sinkpad, gint64 *value)
 {
@@ -1411,7 +1441,6 @@ gst_dspaudio_query_peer_duration(GstPad *sinkpad, gint64 *value)
  *
  * Returns: TRUE, if operation wasa successful. Otherwise FALSE
  */
-
 gboolean
 gst_dspaudio_get_info(GstDSPAudio *dsp)
 {
@@ -1444,8 +1473,8 @@ gst_dspaudio_get_info(GstDSPAudio *dsp)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
-gboolean gst_dspaudio_aep_initialize(GstDSPAudio *dsp)
+gboolean
+gst_dspaudio_aep_initialize(GstDSPAudio *dsp)
 {
   if(gst_dspaudio_open_node(&dsp->aep, AEP_DEVICE_NAME) == FALSE) {
     dsp->mode = DSP_MODE_ERROR;
@@ -1464,8 +1493,8 @@ gboolean gst_dspaudio_aep_initialize(GstDSPAudio *dsp)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
-gboolean gst_dspaudio_aep_close(GstDSPAudio *dsp)
+gboolean
+gst_dspaudio_aep_close(GstDSPAudio *dsp)
 {
   return gst_dspaudio_close_node(&dsp->aep);
 }
@@ -1480,7 +1509,6 @@ gboolean gst_dspaudio_aep_close(GstDSPAudio *dsp)
  *
  * Returns: TRUE if operation was successful
  */
-
 gboolean
 gst_dspaudio_ed_microphone(GstDSPAudio *dsp, gboolean mic_enable)
 {
@@ -1499,53 +1527,125 @@ gst_dspaudio_ed_microphone(GstDSPAudio *dsp, gboolean mic_enable)
 
 
 /**
+ * gst_dspaudio_default_voice_frame_handler:
+ * @mmap_ptr: Pointer to audio frame data
+ * @fsize: Size of the incoming frame, in BYTES
  *
+ * Default handler for voice frames. It just creates a GstBuffer and copies
+ * the audio frame there. Some audio formats (e.g. G.729) need some processing
+ * of the frame before it can be passed forward and they can override this
+ * handler with their own.
  *
+ * Returns: New GstBuffer containing the audio frame
  */
+static GstBuffer *
+gst_dspaudio_default_voice_frame_handler (short int *mmap_ptr, guint fsize)
+{
+    GstBuffer *newbuf = gst_buffer_new_and_alloc(fsize);
+    memcpy(GST_BUFFER_DATA(newbuf), mmap_ptr, fsize);
+    return newbuf;
+}
 
+
+/**
+ * gst_dspaudio_default_sid_frame_handler:
+ * @mmap_ptr: Pointer to SID_UPDATE frame data
+ * @fsize: Size of the incoming frame, in BYTES
+ *
+ * Default handler for SID_UPDATE frames. Converts the DSP frame representation
+ * into G.711 CN frame format. This also sets GST_BUFFER_FLAG_LAST flag in
+ * in the buffer to mark it as CN buffer.
+ *
+ * Returns: New GstBuffer containing the audio frame
+ */
+static GstBuffer *
+gst_dspaudio_default_sid_frame_handler (short int *mmap_ptr, guint fsize)
+{
+/* Currently the coefficients seem to cause distorted noise, so 
+   they are not used. Only the "level" value is sent */
+
+#if 0
+    gint output_cn_size = DSP_ILBC_SID_UPDATE_FRAMESIZE / sizeof (short int);
+    GstBuffer *newbuf = gst_buffer_new_and_alloc (output_cn_size + 2);
+    char *data = GST_BUFFER_DATA (newbuf);
+    gint i;
+
+    /* Do transformation into G.711 appendix II A format */
+    for (i=0; i < output_cn_size; i++) {
+        data[i] = (char) mmap_ptr[i];
+    }
+#else
+    GstBuffer *newbuf = gst_buffer_new_and_alloc (1);
+    char *data = GST_BUFFER_DATA (newbuf);
+    data[0] = mmap_ptr[0]; /* Copy only the level value */
+#endif
+
+    GST_BUFFER_FLAG_SET(newbuf, GST_BUFFER_FLAG_LAST);
+    return newbuf;
+}
+
+
+/**
+ * gst_dspaudio_read_frame:
+ * @dsp: DSPAudio object
+ * @framesize: size of the frame to be read, in BYTES
+ * @ufs: size of the SID_UPDATE frame, in BYTES
+ * @need_cn: TRUE if the plugin wants to generate comfort noise packets
+ *
+ * Read one audio frame from DSP.
+ * 
+ * Returns: GstBuffer containing the frame
+ */
 GstBuffer *
-gst_dspaudio_read_dtx_frame(GstDSPAudio *dsp, guint framesize)
+gst_dspaudio_read_frame(GstDSPAudio *dsp, guint framesize, guint ufs,
+                        gboolean need_cn)
 {
   short int tmp = DSP_CMD_DATA_READ;
   short int *mmap_ptr = (short int *) dsp->codec.mmap_buffer;
-  gboolean found = FALSE;
   GstBuffer *newbuf = NULL;
   DSPWaitStatus status;
-  guint8 *data;
 
   do {
     if(dsp->all_frames_read) {
+      g_mutex_lock(dsp->dsp_mutex);
       status = gst_dspaudio_wait_buffer(dsp);
+      g_mutex_unlock(dsp->dsp_mutex);
       if(status == DSP_WAIT_INTERRUPT) {
-      // Engine is running the pipeline down. We need to exit now
+        // Engine is running the pipeline down. We need to exit now
         DBG_PRINT("READ_DTX_FRAME EXIT\n");
         return NULL;
       }
-      if(status == DSP_WAIT_ERROR) {
+      else if(status == DSP_WAIT_RETRY) {
+        continue;
+      }
+      else if(status == DSP_WAIT_ERROR) {
         dsp->mode = DSP_MODE_ERROR;
         return NULL;
       }
-      DBG_PRINT("FRAMESIZE: %d\n", dsp->readinfo.frame_size);
+//       DBG_PRINT("FRAMESIZE: %d\n", dsp->readinfo.frame_size);
       dsp->all_frames_read = FALSE;
       dsp->readindex = 0;
     }
 
     switch(mmap_ptr[dsp->readindex++]) {
       case VOICE_FRAME:
-//         DBG_PRINT("VOICE FRAME\n");
-        newbuf = gst_buffer_new_and_alloc(framesize);
-        data = GST_BUFFER_DATA(newbuf);
-        memcpy(data, &(mmap_ptr[dsp->readindex]), framesize);
-        GST_BUFFER_SIZE(newbuf) = framesize;
+        //DBG_PRINT("VOICE FRAME\n");
+        newbuf = dsp->handle_voice_frame (&(mmap_ptr[dsp->readindex]), framesize);
         dsp->readindex += (framesize / sizeof(short int));
-        found = TRUE;
         break;
       case SID_FRAME_UPDATE:
-//         DBG_PRINT("SID_FRAME_UPDATE\n");
-        dsp->readindex += 11;
+        //DBG_PRINT("SID_FRAME_UPDATE\n");
+        if (need_cn) {
+            newbuf = dsp->handle_sid_frame (&(mmap_ptr[dsp->readindex]), ufs);
+        }
+        else {
+            newbuf = gst_buffer_new ();
+        }
+        dsp->readindex += (ufs / sizeof(short int));
         break;
       case SID_FRAME_NO_UPDATE:
-//         DBG_PRINT("SID_FRAME_NO_UPDATE\n");
+        //DBG_PRINT("SID_FRAME_NO_UPDATE\n");
+        newbuf = gst_buffer_new ();
         break;
       default:
         DBG_PRINT("Unknown header: %d\n", mmap_ptr[dsp->readindex-1]);
@@ -1559,67 +1659,26 @@ gst_dspaudio_read_dtx_frame(GstDSPAudio *dsp, guint framesize)
       dsp->read_sent = TRUE;
     }
   }
-  while(!found);
+  while (FALSE);
 
   return newbuf;
 }
 
 
 /**
- *
+ * gst_dspaudio_clock_change_state:
  *
  */
-
-GstBuffer *
-gst_dspaudio_read_normal_frame(GstDSPAudio *dsp, guint framesize, guint *received)
-{
-  short int tmp = DSP_CMD_DATA_READ;
-  DSPWaitStatus status;
-  int i;
-
-  status = gst_dspaudio_wait_buffer(dsp);
-  if( status == DSP_WAIT_INTERRUPT ) {
-    *received = 0;
-    return NULL;
-  }
-  if( status == DSP_WAIT_ERROR ) {
-    dsp->mode = DSP_MODE_ERROR;
-    *received = 0;
-    return NULL;
-  }
-
-  guint recv_frames = dsp->readinfo.frame_size / (framesize / sizeof(short int));
-  GstBuffer *newbuf = gst_buffer_new_and_alloc(recv_frames * framesize);
-  guint8 *data = GST_BUFFER_DATA(newbuf);
-
-  DBG_PRINT("Received %d frames (framesize = %d short ints)\n",
-            recv_frames, dsp->readinfo.frame_size);
-
-  for(i=0;i<recv_frames;i++) {
-    memcpy(data + i*framesize,
-           2 + dsp->codec.mmap_buffer + i * (framesize + sizeof(short int)),
-           framesize);
-  }
-  GST_BUFFER_SIZE(newbuf) = recv_frames * framesize;
-
-  if(write(dsp->codec.fd, &tmp, sizeof(short int)) < 0) {
-    dsp->mode = DSP_MODE_ERROR;
-  }
-  *received = recv_frames;
-  return newbuf;
-}
-
-
-/**
- *
- *
- */
-
 GstStateChangeReturn
 gst_dspaudio_clock_change_state(GstDSPAudio *dsp, GstStateChange transition)
 {
   switch (transition) {
 
+  case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+    /* Disable interpolating clock time using system clock *
+     * until next clock time is read from dsp */
+    dsp->last_dsp_update = 0;
+    break;
   case GST_STATE_CHANGE_NULL_TO_READY:
     dsp->clock_time = 0;
     dsp->last_dsp_update = 0;
@@ -1631,12 +1690,12 @@ gst_dspaudio_clock_change_state(GstDSPAudio *dsp, GstStateChange transition)
 }
 
 
-/**
- *
+/*
+ * gst_dspaudio_clock_interrupt:
  *
  */
-
-gboolean gst_dspaudio_clock_interrupt(GstDSPAudio *dsp)
+static gboolean
+gst_dspaudio_clock_interrupt(GstDSPAudio *dsp)
 {
   gboolean ret = FALSE;
 
@@ -1656,13 +1715,12 @@ gboolean gst_dspaudio_clock_interrupt(GstDSPAudio *dsp)
 
 
 /**
- *
+ * gst_dspaudio_get_dsp_time:
  *
  */
-
-GstClockTime
-gst_dspaudio_get_dsp_time(GstDSPAudio *dsp){
-
+static GstClockTime
+gst_dspaudio_get_dsp_time(GstDSPAudio *dsp)
+{
   guint64 time64 = 0;
 
   // If mutex is not locked, we can freely ask the time from DSP
@@ -1691,7 +1749,7 @@ gst_dspaudio_get_dsp_time(GstDSPAudio *dsp){
   time64 |= dsp->info.samples_played_low;
 
   dsp->clock_time = time64 * GST_MSECOND;
-//   DBG_PRINT("DSP CLOCK TIME: %lld (%lld secs)\n", dsp->clock_time, dsp->clock_time/GST_SECOND);
+  //DBG_PRINT("DSP CLOCK TIME: %lld (%lld secs)\n", dsp->clock_time, dsp->clock_time/GST_SECOND);
   return dsp->clock_time;
 
 }
@@ -1705,7 +1763,6 @@ gst_dspaudio_get_dsp_time(GstDSPAudio *dsp){
  *
  * Returns: Clock
  */
-
 GstClockTime
 gst_dspaudio_get_dsp_clock(GstDSPAudio *dsp)
 {
@@ -1718,8 +1775,8 @@ gst_dspaudio_get_dsp_clock(GstDSPAudio *dsp)
     g_get_current_time (&systime);
     systime_abs = GST_TIMEVAL_TO_TIME (systime);
 
-//     DBG_PRINT("DSPAUDIO elapsed time since last update: %lld ms\n",
-//               ((systime_abs - dsp->last_dsp_update)/GST_MSECOND));
+    //DBG_PRINT("DSPAUDIO elapsed time since last update: %lld ms\n",
+    //          ((systime_abs - dsp->last_dsp_update)/GST_MSECOND));
 
     // This function might get called from multiple threads simultaneously
     // so we need to restrict the access to DSP clock reading
@@ -1748,31 +1805,240 @@ gst_dspaudio_get_dsp_clock(GstDSPAudio *dsp)
 }
 
 /**
- * plugin_init:
- * @plugin: GstPlugin to be initialized
- *
- * This initializes the plugin when it is loaded by GStreamer
- *
- * Returns: Boolean value, TRUE if initialization was successful
+ * gst_dspaudio_map_dtmf:
+ * @tone_char: character to be mapped
+ * @tone_id: output parameter for DSP tone ID
+ * 
+ * Map the given character to DSP tone ID
+ * Returns: TRUE if mapping found, otherwise FALSE.
  */
-
-static gboolean
-plugin_init (GstPlugin * plugin)
+gboolean
+gst_dspaudio_map_dtmf(gint tone_id, unsigned short int *dsp_tone_id)
 {
+  gboolean ret = TRUE;
+
+  switch(tone_id) {
+    case 0:  *dsp_tone_id = DTMF_0; break;
+    case 1:  *dsp_tone_id = DTMF_1; break;
+    case 2:  *dsp_tone_id = DTMF_2; break;
+    case 3:  *dsp_tone_id = DTMF_3; break;
+    case 4:  *dsp_tone_id = DTMF_4; break;
+    case 5:  *dsp_tone_id = DTMF_5; break;
+    case 6:  *dsp_tone_id = DTMF_6; break;
+    case 7:  *dsp_tone_id = DTMF_7; break;
+    case 8:  *dsp_tone_id = DTMF_8; break;
+    case 9:  *dsp_tone_id = DTMF_9; break;
+    case 10: *dsp_tone_id = DTMF_STAR; break;
+    case 11: *dsp_tone_id = DTMF_HASH; break;
+    case 12: *dsp_tone_id = DTMF_A; break;
+    case 13: *dsp_tone_id = DTMF_B; break;
+    case 14: *dsp_tone_id = DTMF_C; break;
+    case 15: *dsp_tone_id = DTMF_D; break;
+    default: ret = FALSE; break;
+  }
+  return ret;
+}
+
+
+/**
+ * gst_dspaudio_dtmf_start:
+ * @dsp: DSP audio object
+ * @tone_char: DTMF tone character to be sent
+ *
+ * Send DTMF_START command to DSP
+ *
+ * Returns: TRUE if the operation was successful, otherwise FALSE
+ */
+gboolean
+gst_dspaudio_dtmf_tone_start(GstDSPAudio *dsp, gint tone_id)
+{
+  TONE_STATUS status;
+
+  if(dsp==NULL) return FALSE;
+  if(dsp->codec.fd==-1) return FALSE;
+  if(dsp->mode==DSP_MODE_ERROR) return FALSE;
+
+  if(dsp->mode != DSP_MODE_PLAYING) {
+    GST_DEBUG("not in playing mode\n");
+    return FALSE;
+  }
+
+  if(dsp->dtmf_playing) {
+    // Stop the previous tone before playing new
+    DBG_PRINT("turning off old tone\n");
+    gst_dspaudio_dtmf_tone_stop(dsp);
+  }
+
+  if(gst_dspaudio_map_dtmf(tone_id, &status.tone_id)) {
+    DBG_PRINT("starting DTMF tone: %d\n", status.tone_id);
+    status.dsp_cmd = DSP_CMD_DTMF_START;
+    status.amplitude = 29205; // FIXME: Hardcoded value
+    if(write(dsp->codec.fd, &status, sizeof(TONE_STATUS)) < 0) {
+        dsp->mode = DSP_MODE_ERROR;
+        return FALSE;
+    }
+    gst_dspaudio_read_cmd(dsp, 0);
+    dsp->dtmf_playing = TRUE;
+  }
+  DBG_PRINT("tone command sent\n");
   return TRUE;
 }
 
+
 /**
- * This is the "boilerplate" of the plugin
+ * gst_dspaudio_dtmf_stop:
+ * @dsp: DSP audio object
  *
+ * Send DTMF_STOP command to DSP
+ *
+ * Returns: TRUE if the operation was successful, otherwise FALSE
  */
 
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-                   GST_VERSION_MINOR,
-                   "gstdspaudio",
-                   "Utility plugin for all DSP audio elements",
-                   plugin_init,
-                   VERSION,
-                   "LGPL",
-                   "dspaudio",
-                   "Nokia Corporation")
+gboolean
+gst_dspaudio_dtmf_tone_stop(GstDSPAudio *dsp)
+{
+  DSP_CMD_STATUS status;
+
+  if(dsp==NULL) return FALSE;
+  if(dsp->codec.fd==-1) return FALSE;
+  if(dsp->mode==DSP_MODE_ERROR) return FALSE;
+
+  if(dsp->dtmf_playing) {
+    DBG_PRINT("stopping DTMF tone\n");
+    status.dsp_cmd = DSP_CMD_DTMF_STOP;
+    if(write(dsp->codec.fd, &status, sizeof(short int)) < 0) {
+        dsp->mode = DSP_MODE_ERROR;
+        return FALSE;
+    }
+    dsp->dtmf_playing = FALSE;
+    gst_dspaudio_read_cmd(dsp, 0);
+    DBG_PRINT("DTMF tone stopped\n");
+  }
+  return TRUE;
+}
+
+
+/**
+ * gst_dspaudio_check_upstream_event:
+ * @dsp: DSP audio object
+ * @event: GstEvent to be checked
+ *
+ * Check if the event is DTMF event
+ *
+ * Returns: TRUE if the event was handled, otherwise FALSE.
+ */
+
+gboolean
+gst_dspaudio_check_upstream_event(GstDSPAudio *dsp, GstEvent *event)
+{
+  const GstStructure *structure = gst_event_get_structure (event);
+  if(structure && gst_structure_has_name (structure, "dtmf-event")) {
+    gboolean start = FALSE;
+    guint method = 0;
+    gint tone_id = -1;
+    gint event_type;
+
+    DBG_PRINT("got DTMF event\n");
+    if (gst_structure_get_int (structure, "type", &event_type) &&
+        gst_structure_get_boolean (structure, "start", &start) &&
+        event_type == 1)
+    {
+      if (gst_structure_get_int (structure, "method", &method) &&
+          method != 2)
+      {
+          DBG_PRINT("Wrong DTMF method (not in-band)\n");
+          return FALSE;
+      }
+      g_mutex_lock(dsp->dsp_mutex);
+      if(start) {
+          if(gst_structure_get_int (structure, "number", &tone_id)) {
+              gst_dspaudio_dtmf_tone_start(dsp, tone_id);
+          }
+      }
+      else {
+          gst_dspaudio_dtmf_tone_stop(dsp);
+      }
+      g_mutex_unlock(dsp->dsp_mutex);
+      DBG_PRINT("DTMF event handled\n");
+      return TRUE;
+    }
+  }
+  return FALSE;
+}
+
+
+/*
+ * This function is there so that caps negotiation is done properly
+ */
+gboolean
+gst_dspaudio_activate_push (GstPad * pad, gboolean active)
+{
+  GstCaps *thiscaps;
+  GstCaps *caps = NULL;
+  GstCaps *peercaps = NULL;
+  gboolean result = FALSE;
+
+  DBG_PRINT ("activate push\n");
+
+  if (!active)
+    return TRUE;
+
+  /* first see what is possible on our source pad */
+  thiscaps = gst_pad_get_caps (pad);
+  DBG_PRINT ("caps of src: %" GST_PTR_FORMAT, thiscaps);
+  /* nothing or anything is allowed, we're done */
+  if (thiscaps == NULL)
+    goto no_nego_needed;
+
+  /* get the peer caps */
+  peercaps = gst_pad_peer_get_caps (pad);
+  DBG_PRINT ("caps of peer: %" GST_PTR_FORMAT, peercaps);
+  if (peercaps) {
+    GstCaps *icaps;
+
+    /* get intersection */
+    icaps = gst_caps_intersect (thiscaps, peercaps);
+    DBG_PRINT ("intersect: %" GST_PTR_FORMAT, icaps);
+    gst_caps_unref (thiscaps);
+    gst_caps_unref (peercaps);
+    if (icaps) {
+      /* take first (and best, since they are sorted) possibility */
+      caps = gst_caps_copy_nth (icaps, 0);
+      gst_caps_unref (icaps);
+    }
+  } else {
+    /* no peer, work with our own caps then */
+    caps = thiscaps;
+  }
+  if (caps) {
+    caps = gst_caps_make_writable (caps);
+    gst_caps_truncate (caps);
+
+    /* now fixate */
+    if (!gst_caps_is_empty (caps)) {
+      gst_pad_fixate_caps (pad, caps);
+      DBG_PRINT ("fixated to: %" GST_PTR_FORMAT, caps);
+
+      if (gst_caps_is_any (caps)) {
+        /* hmm, still anything, so element can do anything and
+         * nego is not needed */
+        result = TRUE;
+      } else if (gst_caps_is_fixed (caps)) {
+        /* yay, fixed caps, use those then */
+        gst_pad_set_caps (pad, caps);
+        result = TRUE;
+      }
+    }
+    gst_caps_unref (caps);
+  }
+  return result;
+
+no_nego_needed:
+  {
+    DBG_PRINT ("no negotiation needed");
+    if (thiscaps)
+      gst_caps_unref (thiscaps);
+    return TRUE;
+  }
+}
+
