@@ -28,22 +28,13 @@
 
 #include "gstdspilbcsink.h"
 
-GstElementDetails gst_dspilbcsink_details = GST_ELEMENT_DETAILS ("DSP ILBC Sink",
-    "Sink/Audio",
-    "ILBC audio sink",
-    "Makoto Sugano <makoto.sugano@nokia.com>");
-
-static gchar *devname = "/dev/dsptask/ilbc_dec";
+static const gchar *devname = "/dev/dsptask/ilbc_dec";
 
 static GstStaticPadTemplate dspilbcsink_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ( "audio/x-iLBC, "
-                      "rate = (int) 8000, "
-                      "channels = (int) 1, "
-                      "mode = (int) { 20, 30 }"
-                    ) );
+    GST_STATIC_CAPS ( "audio/x-iLBC, " "mode = (int) { 20, 30 }" ) );
 
 // Function prototypes
 
@@ -62,28 +53,28 @@ static GstStateChangeReturn
 
 // static GstFlowReturn gst_dspilbcsink_preroll(GstBaseSink *sink, GstBuffer * buffer);
 
-/**
- *
- *
- */
 
-void _ilbcsink_do_init()
+#ifdef DEBUG
+static void _do_init()
 {
   DBG_PRINT("ILBCSINK DO_INIT\n");
 }
 
 GST_BOILERPLATE_FULL (GstDSPILBCSink, gst_dspilbcsink, GstBaseSink,
-                      GST_TYPE_BASE_SINK, _ilbcsink_do_init);
+                      GST_TYPE_BASE_SINK, _do_init);
+#else
+GST_BOILERPLATE (GstDSPILBCSink, gst_dspilbcsink, GstBaseSink,
+                 GST_TYPE_BASE_SINK);
+#endif
 
 
-/**
+/*
  * gst_dspilbcsink_dispose:
  * @object: GObject pointer to element to be deleted
  *
  * Deletes ILBC sink element instance. Called automatically by
  * GLib framework when element needs to be disposed.
  */
-
 static void
 gst_dspilbcsink_dispose (GObject * object)
 {
@@ -94,38 +85,41 @@ gst_dspilbcsink_dispose (GObject * object)
     dsp->audio = NULL;
   }
 
-  g_object_unref(G_OBJECT(dsp->adapter));
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
-/**
+/*
  * gst_dspilbcsink_base_init:
  * @g_class: ILBC sink class
  *
  * Does the basic initialization of ILBC sink element class. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspilbcsink_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  static const GstElementDetails details = GST_ELEMENT_DETAILS (
+    "DSP ILBC Sink",
+    "Sink/Audio",
+    "ILBC audio sink",
+    "Makoto Sugano <makoto.sugano@nokia.com>");
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&dspilbcsink_sink_template));
-  gst_element_class_set_details (gstelement_class, &gst_dspilbcsink_details);
+  gst_element_class_set_details (gstelement_class, &details);
 
 }
 
-/**
+
+/*
  * gst_dspilbcsink_class_init:
  * @klass: ILBC Sink class object
  *
  * Initializes ILBC sink element class. This is called by GStreamer
  * framework.
  */
-
 static void
 gst_dspilbcsink_class_init (GstDSPILBCSinkClass * klass)
 {
@@ -155,7 +149,7 @@ gst_dspilbcsink_class_init (GstDSPILBCSinkClass * klass)
 }
 
 
-/**
+/*
  * gst_dspilbcsink_init:
  * @dsp: DSP ILBC sink object
  *
@@ -163,27 +157,28 @@ gst_dspilbcsink_class_init (GstDSPILBCSinkClass * klass)
  * resources. This also creates an audio stream object. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspilbcsink_init (GstDSPILBCSink * dspilbcsink, GstDSPILBCSinkClass *g_class)
 {
   dspilbcsink->audio = gst_dspaudio_new();
-  dspilbcsink->adapter = gst_adapter_new();
+  gst_base_sink_set_sync (GST_BASE_SINK (dspilbcsink), FALSE);
 }
 
 
-/**
+/*
  * gst_dspilbcsink_setcaps:
  * @basesink: GstBaseSink
  * @caps: GstCaps that were offered by the connecting element
  *
  * Returns: TRUE if caps can be accepted, otherwise FALSE
  */
-
 static gboolean
 gst_dspilbcsink_setcaps(GstBaseSink * basesink, GstCaps * caps)
 {
-  DBG_PRINT("gst_dspilbcsink_setcaps\n");
+  gchar *caps_str = gst_caps_to_string (caps);
+  DBG_PRINT("gst_dspilbcsink_setcaps: %s\n", caps_str);
+  g_free (caps_str);
+
   GstDSPILBCSink *dsp = GST_DSPILBCSINK (basesink);
   SPEECH_PARAMS_DATA init_data;
 
@@ -213,38 +208,23 @@ gst_dspilbcsink_setcaps(GstBaseSink * basesink, GstCaps * caps)
 }
 
 
-/**
+/*
  * gst_dspilbcsink_render:
  * sink:
  * @buffer:
  *
  */
-
-#ifdef DEBUG
-int fd = -1;
-#endif
-
 static GstFlowReturn
 gst_dspilbcsink_render(GstBaseSink *sink, GstBuffer * buffer)
 {
   GstFlowReturn ret = GST_FLOW_OK;
   GstDSPILBCSink *dsp = GST_DSPILBCSINK (sink);
+  const guint8 *data = GST_BUFFER_DATA (buffer);
+  guint avail = GST_BUFFER_SIZE (buffer);
   DSPWaitStatus status;
-  const guint8 *data;
-  guint avail;
+  guint dataIndex = 0;
 
-  DBG_PRINT("ILBC RENDER: %d bytes\n", GST_BUFFER_SIZE(buffer));
-
-  gst_buffer_ref(buffer);
-  gst_adapter_push (dsp->adapter, buffer);
-
-#ifdef DEBUG
-/*  if(fd == -1) {
-    fd = open("/home/user/debug.out", O_RDWR | O_CREAT);
-    DBG_PRINT("OPENED DEBUG FILE\n");
-  }
-  write(fd, GST_BUFFER_DATA(buffer), GST_BUFFER_SIZE(buffer));*/
-#endif
+  DBG_PRINT("ILBC RENDER: %d bytes\n", avail);
 
   if(dsp->audio->mode != DSP_MODE_PLAYING) {
     DBG_PRINT("SETTING DSP TO PLAY\n");
@@ -255,11 +235,9 @@ gst_dspilbcsink_render(GstBaseSink *sink, GstBuffer * buffer)
     gst_dspaudio_update_dsp_settings(dsp->audio);
   }
 
-  avail = gst_adapter_available(dsp->adapter);
   while(avail >= dsp->framesize) {
 
     g_mutex_lock(dsp->audio->dsp_mutex);
-
     status = gst_dspaudio_wait_buffer(dsp->audio);
 
     if(status == DSP_WAIT_RETRY) {
@@ -283,18 +261,18 @@ gst_dspilbcsink_render(GstBaseSink *sink, GstBuffer * buffer)
     }
 
     dsp->audio->rw_pending = 1;
-    data = gst_adapter_peek(dsp->adapter, dsp->framesize);
-    dsp->outbuf.data_size = dsp->framesize / sizeof(short int);
-    avail -= dsp->framesize;
+    dsp->outbuf.data_size = 3 + (dsp->framesize / sizeof(short int));
 
     dsp->mmap_ptr[0] = 0;             // Bad frame indication
     dsp->mmap_ptr[1] = 0;             // Last frame indication
     dsp->mmap_ptr[2] = VOICE_FRAME;   // Frame type
 
-    memcpy(&(dsp->mmap_ptr[3]), data, dsp->framesize);
-    gst_adapter_flush(dsp->adapter, dsp->framesize);
+    memcpy(&(dsp->mmap_ptr[3]), data+dataIndex, dsp->framesize);
+    avail -= dsp->framesize;
+    dataIndex += dsp->framesize;
 
-    if(write(dsp->audio->codec.fd, &(dsp->outbuf), sizeof(ILBC_DATA_WRITE)) < 0) {
+    if(write(dsp->audio->codec.fd, &(dsp->outbuf), sizeof(ILBC_DATA_WRITE)) < 0)
+    {
       dsp->audio->mode = DSP_MODE_ERROR;
       GST_ELEMENT_ERROR (dsp, RESOURCE, WRITE, (NULL), ("write = -1"));
       g_mutex_unlock(dsp->audio->dsp_mutex);
@@ -302,22 +280,19 @@ gst_dspilbcsink_render(GstBaseSink *sink, GstBuffer * buffer)
       break;
     }
 
-//     DBG_PRINT("ILBC: Wrote %d bytes to DSP\n", dsp->outbuf.data_size * sizeof(short int));
     dsp->audio->rw_pending = 0;
     g_mutex_unlock(dsp->audio->dsp_mutex);
   }
-  DBG_PRINT("ILBC SINK RENDER EXIT\n");
   return ret;
 }
 
 
-/**
+/*
  * gst_dspilbcsink_preroll:
  * @sink:
  * @buffer:
  *
  */
-
 /*
 static GstFlowReturn
 gst_dspilbcsink_preroll(GstBaseSink *sink, GstBuffer * buffer)
@@ -331,12 +306,12 @@ gst_dspilbcsink_preroll(GstBaseSink *sink, GstBuffer * buffer)
 }
 */
 
-/**
+
+/*
  * gst_dspilbcsink_start:
  * @bsink: GstBaseSink
  *
  */
-
 static gboolean
 gst_dspilbcsink_start (GstBaseSink * bsink)
 {
@@ -356,12 +331,11 @@ gst_dspilbcsink_start (GstBaseSink * bsink)
 }
 
 
-/**
+/*
  * gst_dspilbcsink_stop:
  * @bsink: GstBaseSink
  *
  */
-
 static gboolean
 gst_dspilbcsink_stop (GstBaseSink * bsink)
 {
@@ -379,24 +353,14 @@ gst_dspilbcsink_stop (GstBaseSink * bsink)
   }
   gst_dspaudio_close(dsp->audio);
   gst_dspaudio_reset(dsp->audio);
-  gst_adapter_clear(dsp->adapter);
   g_mutex_unlock(dsp->audio->dsp_mutex);
 
-#ifdef DEBUG
-/*  if(fd != -1) {
-    close(fd);
-    fd = -1;
-  }*/
-#endif
-
   DBG_PRINT("gst_dspilbcsink_stop OK\n");
   return TRUE;
 }
 
 
-
-
-/**
+/*
  * gst_dspilbcsink_set_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -405,7 +369,6 @@ gst_dspilbcsink_stop (GstBaseSink * bsink)
  *
  * Stores the property value into GObject
  */
-
 static void
 gst_dspilbcsink_set_property(GObject * object,
                             guint prop_id,
@@ -425,7 +388,7 @@ gst_dspilbcsink_set_property(GObject * object,
 }
 
 
-/**
+/*
  * gst_dspilbcsink_get_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -434,7 +397,6 @@ gst_dspilbcsink_set_property(GObject * object,
  *
  * Retrieves the desired parameter value from object
  */
-
 static void
 gst_dspilbcsink_get_property(GObject * object,
                             guint prop_id,
@@ -450,11 +412,10 @@ gst_dspilbcsink_get_property(GObject * object,
 }
 
 
-/**
- *
+/*
+ * gst_dspilbcsink_change_state:
  *
  */
-
 static GstStateChangeReturn
 gst_dspilbcsink_change_state (GstElement * element, GstStateChange transition)
 {
