@@ -24,15 +24,13 @@
 #include <unistd.h>
 #include <string.h>
 #include <gst/gst.h>
+#include <gst/audio/gstaudioclock.h>
 
 #include "gstdspilbcsrc.h"
 
-GstElementDetails gst_dspilbcsrc_details = GST_ELEMENT_DETAILS ("DSP ILBC Src",
-    "Source/Audio",
-    "ILBC audio src",
-    "Makoto Sugano <makoto.sugano@nokia.com>");
+#define DSP_ILBC_SID_UPDATE_FRAMESIZE 22
 
-static gchar *devname = "/dev/dsptask/ilbc_enc";
+static const gchar *devname = "/dev/dsptask/ilbc_enc";
 
 static GstStaticPadTemplate dspilbcsrc_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -44,6 +42,16 @@ GST_STATIC_PAD_TEMPLATE ("src",
                       "mode = (int) { 20, 30 }"
                     ) );
 
+static GstStaticPadTemplate dspilbcsrc_cnsrc_template =
+GST_STATIC_PAD_TEMPLATE ("cnsrc%d",
+    GST_PAD_SRC,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS ( "audio/CN, "
+                      "rate = (int) 8000, "
+                      "channels = (int) 1"
+                    ) );
+
+
 // Function prototypes
 
 static void gst_dspilbcsrc_class_init (GstDSPILBCSrcClass * klass);
@@ -54,28 +62,34 @@ static void gst_dspilbcsrc_get_property (GObject * object, guint prop_id, GValue
 
 static GstCaps *gst_dspilbcsrc_getcaps (GstBaseSrc * src);
 static gboolean gst_dspilbcsrc_setcaps (GstBaseSrc * src, GstCaps *caps);
+static GstClock *gst_dspilbcsrc_provide_clock (GstElement * elem);
+static GstClockTime gst_dspilbcsrc_get_time (GstClock * clock, GstDSPILBCSrc * src);
 
 static GstFlowReturn gst_dspilbcsrc_create (GstPushSrc * psrc, GstBuffer ** buf);
 static gboolean gst_dspilbcsrc_start (GstBaseSrc * bsrc);
 static gboolean gst_dspilbcsrc_stop (GstBaseSrc * bsrc);
+static gboolean gst_dspilbcsrc_event (GstBaseSrc * bsrc, GstEvent *event);
 
 static GstStateChangeReturn
 gst_dspilbcsrc_change_state (GstElement * element, GstStateChange transition);
 
+static GstPad *gst_dspilbcsrc_request_new_pad (GstElement *element, GstPadTemplate *templ,
+                                               const gchar *name);
+static void gst_dspilbcsrc_release_pad (GstElement *element, GstPad *pad);
 
-/**
- *
- *
- */
 
-void _ilbcsrc_do_init()
+#ifdef DEBUG
+static void _do_init()
 {
   DBG_PRINT("ILBCSRC DO_INIT\n");
 }
 
 GST_BOILERPLATE_FULL (GstDSPILBCSrc, gst_dspilbcsrc, GstPushSrc,
-                      GST_TYPE_PUSH_SRC, _ilbcsrc_do_init);
-
+                      GST_TYPE_PUSH_SRC, _do_init);
+#else
+GST_BOILERPLATE (GstDSPILBCSrc, gst_dspilbcsrc, GstPushSrc,
+                 GST_TYPE_PUSH_SRC);
+#endif
 
 
 #define GST_TYPE_ILBCSRC_MODE (gst_dspilbcsrc_mode_get_type())
@@ -89,82 +103,78 @@ gst_dspilbcsrc_mode_get_type (void)
     {0, NULL, NULL},
   };
 
-  if (!dspilbcsrc_mode_type) {
+  if (G_UNLIKELY(!dspilbcsrc_mode_type)) {
     dspilbcsrc_mode_type = g_enum_register_static ("GstILBCSrcMode",
         dspilbcsrc_modes);
   }
   return dspilbcsrc_mode_type;
 }
 
-#define GST_TYPE_ILBCSRC_DTX_MODE (gst_dspilbcsrc_dtx_mode_get_type())
-static GType
-gst_dspilbcsrc_dtx_mode_get_type (void)
-{
-  static GType dspilbcsrc_dtx_mode_type = 0;
-  static GEnumValue dspilbcsrc_dtx_modes[] = {
-    {GST_ILBCSRC_DTX_MODE_OFF, "Don't use DTX", "off"},
-    {GST_ILBCSRC_DTX_MODE_ON, "Use DTX", "on"},
-    {0, NULL, NULL},
-  };
 
-  if (!dspilbcsrc_dtx_mode_type) {
-    dspilbcsrc_dtx_mode_type = g_enum_register_static ("GstILBCSrcDTXMode",
-        dspilbcsrc_dtx_modes);
-  }
-  return dspilbcsrc_dtx_mode_type;
-}
-
-
-/**
+/*
  * gst_dspilbcsrc_dispose:
  * @object: GObject pointer to element to be deleted
  *
  * Deletes ILBC src element instance. Called automatically by
  * GLib framework when element needs to be disposed.
  */
-
 static void
 gst_dspilbcsrc_dispose (GObject * object)
 {
   GstDSPILBCSrc *dsp = (GstDSPILBCSrc *) object;
 
+  if(dsp->clock) {
+    gst_object_unref(dsp->clock);
+    dsp->clock = NULL;
+  }
+
   if(dsp->audio) {
     gst_dspaudio_destroy(dsp->audio);
     dsp->audio = NULL;
   }
 
+  if (dsp->lastcn) {
+    gst_buffer_unref (dsp->lastcn);
+    dsp->lastcn = NULL;
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
-/**
+/*
  * gst_dspilbcsrc_base_init:
  * @g_class: ILBC src class
  *
  * Does the basic initialization of ILBC src element class. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspilbcsrc_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  static const GstElementDetails details = GST_ELEMENT_DETAILS (
+    "DSP ILBC Src",
+    "Source/Audio",
+    "ILBC audio src",
+    "Makoto Sugano <makoto.sugano@nokia.com>");
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&dspilbcsrc_src_template));
-  gst_element_class_set_details (gstelement_class, &gst_dspilbcsrc_details);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&dspilbcsrc_cnsrc_template));
+  gst_element_class_set_details (gstelement_class, &details);
 
 }
 
 
-/**
+/*
  * gst_dspilbcsrc_class_init:
  * @klass: ILBC Src class object
  *
  * Initializes ILBC src element class. This is called by GStreamer
  * framework.
  */
-
 static void
 gst_dspilbcsrc_class_init (GstDSPILBCSrcClass * klass)
 {
@@ -195,22 +205,26 @@ gst_dspilbcsrc_class_init (GstDSPILBCSrcClass * klass)
   g_object_class_install_property (gobject_class, DSPILBC_PROP_DTX_MODE,
                                    g_param_spec_enum ("dtx", "DTX mode",
                                        "Discontinuous transmission mode",
-                                       GST_TYPE_ILBCSRC_DTX_MODE,        /* enum type */
-                                       GST_ILBCSRC_DTX_MODE_ON,          /* default value */
+                                       GST_TYPE_DSPAUDIO_DTX_MODE,   /* enum type */
+                                       GST_DSPAUDIO_DTX_MODE_ON,     /* default value */
                                        G_PARAM_READWRITE));
 
-  gstelement_class->change_state = gst_dspilbcsrc_change_state;
+  gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_change_state);
+  gstelement_class->provide_clock = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_provide_clock);
+  gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR(gst_dspilbcsrc_request_new_pad);
+  gstelement_class->release_pad = GST_DEBUG_FUNCPTR(gst_dspilbcsrc_release_pad);
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_stop);
   gstbasesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_getcaps);
   gstbasesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_setcaps);
+  gstbasesrc_class->event = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_event);
 
   gstpushsrc_class->create = GST_DEBUG_FUNCPTR (gst_dspilbcsrc_create);
 }
 
 
-/**
+/*
  * gst_dspilbcsrc_init:
  * @dsp: DSP ILBC src object
  *
@@ -218,22 +232,25 @@ gst_dspilbcsrc_class_init (GstDSPILBCSrcClass * klass)
  * resources. This also creates an audio stream object. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspilbcsrc_init (GstDSPILBCSrc * dspilbcsrc, GstDSPILBCSrcClass *g_class)
 {
   dspilbcsrc->audio = gst_dspaudio_new();
-  dspilbcsrc->dtxmode = GST_ILBCSRC_DTX_MODE_ON;
+  gst_base_src_set_live (GST_BASE_SRC (dspilbcsrc), TRUE);
+  dspilbcsrc->dtxmode = GST_DSPAUDIO_DTX_MODE_ON;
   dspilbcsrc->mode = GST_ILBCSRC_MODE_20;
   dspilbcsrc->frametimemillis = 20;
+
+  dspilbcsrc->clock = gst_audio_clock_new ("clock",
+                                           (GstAudioClockGetTimeFunc) gst_dspilbcsrc_get_time,
+                                           dspilbcsrc);
 }
 
 
-/**
- *
+/*
+ * gst_dspilbcsrc_getcaps:
  *
  */
-
 static GstCaps *
 gst_dspilbcsrc_getcaps (GstBaseSrc * src)
 {
@@ -249,11 +266,10 @@ gst_dspilbcsrc_getcaps (GstBaseSrc * src)
 }
 
 
-/**
- *
+/*
+ * gst_dspilbcsrc_setcaps:
  *
  */
-
 static gboolean
 gst_dspilbcsrc_setcaps (GstBaseSrc * src, GstCaps *caps)
 {
@@ -275,18 +291,16 @@ gst_dspilbcsrc_setcaps (GstBaseSrc * src, GstCaps *caps)
 }
 
 
-/**
+/*
  * gst_dspilbcsrc_create:
  * @psrc: GstPushSrc
  * @buf: GstBuffer
  *
  */
-
 static GstFlowReturn
 gst_dspilbcsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 {
   GstDSPILBCSrc *dsp = GST_DSPILBCSRC(psrc);
-  guint recv_frames = 1;
 
   if(dsp->audio->mode == DSP_MODE_ERROR) {
     GST_ELEMENT_ERROR (dsp, RESOURCE, READ, (NULL),
@@ -296,31 +310,59 @@ gst_dspilbcsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
     return GST_FLOW_ERROR;
   }
 
-  if(dsp->dtxmode == GST_ILBCSRC_DTX_MODE_ON) {
-    *buf = gst_dspaudio_read_dtx_frame(dsp->audio, dsp->framesize);
-  }
-  else {
-    *buf = gst_dspaudio_read_normal_frame(dsp->audio, dsp->framesize, &recv_frames);
-  }
+  for (;;) {
+    *buf = gst_dspaudio_read_frame(dsp->audio, dsp->framesize,
+                                   DSP_ILBC_SID_UPDATE_FRAMESIZE, TRUE);
 
-  if(*buf) {
-    GST_BUFFER_DURATION(*buf) = recv_frames * dsp->frametimemillis * GST_MSECOND;
+    if (!*buf)
+      return GST_FLOW_WRONG_STATE;
+
+    GST_BUFFER_DURATION(*buf) = dsp->frametimemillis * GST_MSECOND;
     GST_BUFFER_TIMESTAMP(*buf) = dsp->framecount * dsp->frametimemillis * GST_MSECOND;
-    gst_buffer_set_caps (*buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
-    dsp->framecount += recv_frames;
-    DBG_PRINT("ILBC SRC CREATED A BUFFER\n");
-    return GST_FLOW_OK;
+    dsp->framecount++;
+
+    /* SID_FRAME_NO_UPDATE: try to resend old CN frame */
+    if (GST_BUFFER_SIZE(*buf) == 0) {
+      gst_buffer_unref(*buf);
+      if (dsp->lastcn && dsp->cnpad) {
+        gst_buffer_ref(dsp->lastcn);
+        *buf = dsp->lastcn;
+      } else {
+        continue;
+      }
+    } else if (GST_BUFFER_FLAG_IS_SET(*buf, GST_BUFFER_FLAG_LAST)) {
+      /* is SID_FRAME_UPDATE */
+      if (!dsp->cnpad) {
+        gst_buffer_unref(*buf);
+        continue;
+      }
+      /* Save for future NO_UPDATE */
+      if (dsp->lastcn)
+        gst_buffer_unref(dsp->lastcn);
+      gst_buffer_ref(*buf);
+      dsp->lastcn = *buf;
+      gst_buffer_set_caps (dsp->lastcn, GST_PAD_CAPS (dsp->cnpad));
+    }
+
+    if (GST_BUFFER_CAPS(*buf)) {
+      /* CN Buffer */
+      GST_BUFFER_FLAG_UNSET(*buf, GST_BUFFER_FLAG_LAST);
+      gst_pad_push(dsp->cnpad, *buf);
+      continue;
+    } else {
+      /* Regular buffer */
+      gst_buffer_set_caps (*buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
+      return GST_FLOW_OK;
+    }
   }
-  return GST_FLOW_WRONG_STATE;
 }
 
 
-/**
+/*
  * gst_dspilbcsrc_start:
  * @bsrc: GstBaseSrc
  *
  */
-
 static gboolean
 gst_dspilbcsrc_start (GstBaseSrc * bsrc)
 {
@@ -352,7 +394,7 @@ gst_dspilbcsrc_start (GstBaseSrc * bsrc)
   }
 
   init_data.dsp_cmd = DSP_CMD_SET_SPEECH_PARAMS;
-  if(dsp->dtxmode == GST_ILBCSRC_DTX_MODE_ON) {
+  if(dsp->dtxmode == GST_DSPAUDIO_DTX_MODE_ON) {
     init_data.audio_fmt = DSP_AFMT_ILBC_DTX;
     DBG_PRINT("DTX ON\n");
   }
@@ -380,17 +422,16 @@ gst_dspilbcsrc_start (GstBaseSrc * bsrc)
   gst_dspaudio_play(dsp->audio);
   gst_dspaudio_update_dsp_settings(dsp->audio);
   dsp->framecount = 0;
+  dsp->lastcn = NULL;
   DBG_PRINT("gst_dspilbcsrc_start OK\n");
   return TRUE;
 }
 
 
-/**
- *
+/*
+ * gst_dspilbcsrc_stop:
  *
  */
-
-
 static gboolean
 gst_dspilbcsrc_stop (GstBaseSrc * bsrc)
 {
@@ -409,8 +450,10 @@ gst_dspilbcsrc_stop (GstBaseSrc * bsrc)
     DBG_PRINT("Got it\n");
   }
 
+  gst_dspaudio_dtmf_tone_stop(dsp->audio);
+
   if(gst_dspaudio_stop(dsp->audio)) {
-    gst_dspaudio_read_cmd(dsp->audio, 200);
+    gst_dspaudio_read_cmd(dsp->audio, 500);
   }
 
   gst_dspaudio_ed_microphone(dsp->audio, FALSE);
@@ -424,7 +467,7 @@ gst_dspilbcsrc_stop (GstBaseSrc * bsrc)
 }
 
 
-/**
+/*
  * gst_dspilbcsrc_set_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -433,7 +476,6 @@ gst_dspilbcsrc_stop (GstBaseSrc * bsrc)
  *
  * Stores the property value into GObject
  */
-
 static void
 gst_dspilbcsrc_set_property(GObject * object,
                             guint prop_id,
@@ -446,7 +488,6 @@ gst_dspilbcsrc_set_property(GObject * object,
     DBG_PRINT("ILBC_SRC MUTE SETTING\n");
   }
 
-
   if(prop_id == DSPILBC_PROP_MODE) {
     dsp->mode = g_value_get_enum(value);
     dsp->framesize = dsp->mode == GST_ILBCSRC_MODE_20 ? 38 : 50;
@@ -463,7 +504,7 @@ gst_dspilbcsrc_set_property(GObject * object,
 }
 
 
-/**
+/*
  * gst_dspilbcsrc_get_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -472,7 +513,6 @@ gst_dspilbcsrc_set_property(GObject * object,
  *
  * Retrieves the desired parameter value from object
  */
-
 static void
 gst_dspilbcsrc_get_property(GObject * object,
                             guint prop_id,
@@ -494,16 +534,65 @@ gst_dspilbcsrc_get_property(GObject * object,
 }
 
 
-/**
+/*
+ * gst_dspilbcsrc_event:
+ * @src: GstBaseSrc object
+ * @event: GstEvent to be handled
+ *
+ * Handles incoming events. Checks if the event is special "custom upstream
+ * event" and in this case forwards it to dspaudio library for checking if
+ * it is DTMF event.
+ *
+ * Returns: TRUE if the event was handled.
+ */
+static gboolean
+gst_dspilbcsrc_event(GstBaseSrc *src, GstEvent *event)
+{
+  GstEventType type = event ? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;
+  GstDSPILBCSrc *dsp = GST_DSPILBCSRC (src);
+  gboolean ret = FALSE;
+
+  if(type == GST_EVENT_CUSTOM_UPSTREAM) {
+    ret = gst_dspaudio_check_upstream_event (dsp->audio, event);
+  }
+  return ret;
+}
+
+
+/*
  *
  *
  */
+static GstClock *
+gst_dspilbcsrc_provide_clock (GstElement * elem)
+{
+  DBG_PRINT("gst_dspilbcsrc_provide_clock\n");
+  GstDSPILBCSrc *dsp = GST_DSPILBCSRC (elem);
+  return GST_CLOCK_CAST (gst_object_ref (dsp->clock));
+}
 
+
+/*
+ *
+ *
+ */
+static GstClockTime
+gst_dspilbcsrc_get_time (GstClock * clock, GstDSPILBCSrc * src)
+{
+  GstClockTime result;
+  result = src->framecount * src->frametimemillis * GST_MSECOND;
+  return result;
+}
+
+
+/*
+ * gst_dspilbcsrc_change_state:
+ *
+ */
 static GstStateChangeReturn
 gst_dspilbcsrc_change_state (GstElement * element, GstStateChange transition)
 {
   GstDSPILBCSrc *dsp = GST_DSPILBCSRC (element);
-  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -514,20 +603,42 @@ gst_dspilbcsrc_change_state (GstElement * element, GstStateChange transition)
     default:
       break;
   }
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-/*
-    switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      gst_dspaudio_remove_interrupt(dsp->audio);
-      break;
-
-    default:
-      break;
-  }
-*/
-
-  return ret;
+  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 }
 
 
+/*
+ * gst_dspilbcsrc_request_new_pad:
+ *
+ */
+static GstPad *
+gst_dspilbcsrc_request_new_pad (GstElement     *element,
+                                GstPadTemplate *templ,
+                                const gchar    *name)
+{
+  GstDSPILBCSrc *dsp = GST_DSPILBCSRC(element);
+
+  DBG_PRINT ("request pad\n");
+
+  if (dsp->cnpad)
+    return NULL;
+
+  dsp->cnpad = gst_pad_new_from_template (templ, name);
+  gst_pad_set_activatepush_function (dsp->cnpad, gst_dspaudio_activate_push);
+  gst_element_add_pad (element, dsp->cnpad);
+  return dsp->cnpad;
+}
+
+
+/*
+ * gst_dspilbcsrc_release_pad:
+ *
+ */
+static void
+gst_dspilbcsrc_release_pad (GstElement *element, GstPad *pad)
+{
+  GstDSPILBCSrc *dsp = GST_DSPILBCSRC(element);
+
+  if (pad == dsp->cnpad)
+    dsp->cnpad = NULL;
+}
