@@ -27,24 +27,19 @@
 
 #include "gstdspmp3sink.h"
 
-GstElementDetails gst_dspmp3sink_details = GST_ELEMENT_DETAILS (
-    "DSP MP3 Sink",
-    "Sink/Audio",
-    "MP3 audio sink",
-    "Makoto Sugano <makoto.sugano@nokia.com>>");
 
-static gchar *dsp_node_names[3] =
+static const gchar *dsp_node_names[3] =
 {
   NULL,
   "/dev/dsptask/mp2dec",
   "/dev/dsptask/mp3dec"
 };
 
-gboolean char_in_tmp = FALSE;
-char tmp_char;
+static gboolean char_in_tmp = FALSE;
+static char tmp_char;
 
 // MP3 bitrate lookup tables
-static guint bitrates[2][3][16] =
+static const guint bitrates[2][3][16] =
 {
   {
     {0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0},
@@ -58,7 +53,7 @@ static guint bitrates[2][3][16] =
   }
 };
 
-static guint mp3_samplerates[4][3] =
+static const guint mp3_samplerates[4][3] =
 {
   { 11025, 12000,  8000, },   // MPEG version 2.5
   {     0,     0,     0, },   // Reserved
@@ -66,7 +61,7 @@ static guint mp3_samplerates[4][3] =
   { 44100, 48000, 32000, }    // MPEG version 1
 };
 
-static int bs[3] = {384, 1152, 1152};
+static const int bs[3] = {384, 1152, 1152};
 
 static guint gst_dspmp3sink_signals[DSPMP3_SIGNAL_LAST] = { 0 };
 
@@ -120,11 +115,7 @@ static GstFlowReturn gst_dspmp3sink_preroll(GstBaseSink *sink, GstBuffer * buffe
 static GstStateChangeReturn gst_dspmp3sink_async_play(GstBaseSink *sink);
 
 
-/**
- *
- *
- */
-
+#ifdef DEBUG
 void _do_init()
 {
   DBG_PRINT("MP3 DO_INIT\n");
@@ -132,23 +123,27 @@ void _do_init()
 
 GST_BOILERPLATE_FULL (GstDSPMP3Sink, gst_dspmp3sink, GstBaseSink,
                       GST_TYPE_BASE_SINK, _do_init);
+#else
+GST_BOILERPLATE (GstDSPMP3Sink, gst_dspmp3sink, GstBaseSink,
+                 GST_TYPE_BASE_SINK);
+#endif
 
-/**
+
+/*
  * gst_dspmp3sink_dispose:
  * @object: GObject pointer to element to be deleted
  *
  * Deletes MP3 sink element instance. Called automatically by
  * GLib framework when element needs to be disposed.
  */
-
 static void
 gst_dspmp3sink_dispose (GObject * object)
 {
   GstDSPMP3Sink *dsp = (GstDSPMP3Sink *) object;
 
-  if(dsp->audio->clock) {
-    gst_object_unref(dsp->audio->clock);
-    dsp->audio->clock = NULL;
+  if(dsp->clock) {
+    gst_object_unref(dsp->clock);
+    dsp->clock = NULL;
   }
 
   if(dsp->audio) {
@@ -161,32 +156,36 @@ gst_dspmp3sink_dispose (GObject * object)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_base_init:
  * @g_class: MP3 sink class
  *
  * Does the basic initialization of MP3 sink element class. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspmp3sink_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  static const GstElementDetails details = GST_ELEMENT_DETAILS (
+    "DSP MP3 Sink",
+    "Sink/Audio",
+    "MP3 audio sink",
+    "Makoto Sugano <makoto.sugano@nokia.com>>");
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&dspmp3sink_sink_template));
-  gst_element_class_set_details (gstelement_class, &gst_dspmp3sink_details);
+  gst_element_class_set_details (gstelement_class, &details);
 }
 
-/**
+
+/*
  * gst_dspmp3sink_class_init:
  * @klass: MP3 Sink class object
  *
  * Initializes MP3 sink element class. This is called by GStreamer
  * framework.
  */
-
 static void
 gst_dspmp3sink_class_init (GstDSPMP3SinkClass * klass)
 {
@@ -225,7 +224,7 @@ gst_dspmp3sink_class_init (GstDSPMP3SinkClass * klass)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_init:
  * @dsp: DSP MP3 sink object
  *
@@ -233,30 +232,28 @@ gst_dspmp3sink_class_init (GstDSPMP3SinkClass * klass)
  * resources. This also creates an audio stream object. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dspmp3sink_init (GstDSPMP3Sink * dspmp3sink, GstDSPMP3SinkClass *g_class)
 {
   GstBaseSink *bsink = GST_BASE_SINK(dspmp3sink);
-  bsink->sync = FALSE;
+  gst_base_sink_set_sync (bsink, FALSE);
   dspmp3sink->sinkpad = bsink->sinkpad;
 
   dspmp3sink->audio = gst_dspaudio_new();
   dspmp3sink->adapter = gst_adapter_new();
 
-  dspmp3sink->audio->clock = gst_audio_clock_new ("clock",
-                                                  (GstAudioClockGetTimeFunc) gst_dspmp3sink_get_time,
-                                                  dspmp3sink);
+  dspmp3sink->clock = gst_audio_clock_new ("clock",
+                                           (GstAudioClockGetTimeFunc) gst_dspmp3sink_get_time,
+                                           dspmp3sink);
 }
 
 
-/**
+/*
  * Utility method for handling the troublesome odd character
  * in the end of MP3 frame (MMAP buffer must always be filled
  * with even number of data, so we must store the last odd
  * byte).
  */
-
 static inline void
 gst_dspmp3sink_get_missing_char(GstDSPMP3Sink *dsp)
 {
@@ -269,13 +266,14 @@ gst_dspmp3sink_get_missing_char(GstDSPMP3Sink *dsp)
 }
 
 
-/**
+/*
+ * gst_dspmp3sink_bpf:
+ *
  * Calculate Bits-per-Second (?) value
  *
  */
-
 gdouble
-gst_dspmp3sink_bpf(MP3Frame *frame, guint bitrate)
+static gst_dspmp3sink_bpf(MP3Frame *frame, guint bitrate)
 {
   // Calculate "bytes-per-frame" value
   gdouble bpf = bitrate;
@@ -300,12 +298,13 @@ gst_dspmp3sink_bpf(MP3Frame *frame, guint bitrate)
 }
 
 
-/**
+/*
+ * gst_dspmp3sink_framesize:
+ *
  * Calculate frame size in bytes including header.
  *
  */
-
-int
+static int
 gst_dspmp3sink_framesize(MP3Frame *frame)
 {
   // Calculate "bytes-per-frame" value
@@ -333,12 +332,12 @@ gst_dspmp3sink_framesize(MP3Frame *frame)
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_send_discont:
  *
  */
-
-static void gst_dspmp3sink_send_discont(GstDSPMP3Sink *dsp)
+static void
+gst_dspmp3sink_send_discont(GstDSPMP3Sink *dsp)
 {
   if(dsp->total_bytes > 0)
   {
@@ -354,9 +353,7 @@ static void gst_dspmp3sink_send_discont(GstDSPMP3Sink *dsp)
 }
 
 
-
-
-/**
+/*
  * gst_dspmp3sink_head_check:
  * @head: MP3 header in long int format
  *
@@ -391,11 +388,13 @@ gboolean gst_dspmp3sink_head_check(guint32 head)
 }
 
 
-static inline guint32 gst_dspmp3sink_convert_to_header(const guint8 * buf) {
+static inline guint32
+gst_dspmp3sink_convert_to_header(const guint8 * buf) {
   return (buf[0] << 24) + (buf[1] << 16) + (buf[2] << 8) + buf[3];
 }
 
-/**
+
+/*
  * gst_dspmp3sink_find_sync:
  * @dsp: DSP MP3 sink object
  *
@@ -403,7 +402,6 @@ static inline guint32 gst_dspmp3sink_convert_to_header(const guint8 * buf) {
  * data stream. This key is can be used as a synchronization
  * marker, since it is a frame boundary identifier.
  */
-
 static guint32
 gst_dspmp3sink_find_sync(GstDSPMP3Sink *dsp, gboolean *gap_detected)
 {
@@ -431,19 +429,21 @@ gst_dspmp3sink_find_sync(GstDSPMP3Sink *dsp, gboolean *gap_detected)
 }
 
 
-/**
+/*
+ * gst_dspmp3sink_mp3_xing_check:
+ * @dsp: DSP MP3 sink object
+ * @data: the buffer containing first frame of the MP3 stream
+ *
  * Checks if the header contains Xing extension. If so, extract the
  * number of frames.
  *
- * @dsp: DSP MP3 sink object
- * @data: the buffer containing first frame of the MP3 stream
  * Returns: Number of frames information, or 0 if no Xing header found
  */
 
 #define GET_INT32BE(b) \
   (i = (b[0] << 24) | (b[1] << 16) | b[2] << 8 | b[3], b += 4, i)
 
-gboolean
+static gboolean
 gst_dspmp3sink_mp3_xing_check(MP3Frame *frame, XINGTag *tag, gchar *data)
 {
   guint i, head_flags;
@@ -509,12 +509,11 @@ gst_dspmp3sink_mp3_xing_check(MP3Frame *frame, XINGTag *tag, gchar *data)
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_mp3_parse_head:
  *
  */
-
-void
+static void
 gst_dspmp3sink_mp3_parse_head(MP3Frame *frame, guint32 head)
 {
   short int br_index, sr_index;
@@ -538,12 +537,11 @@ gst_dspmp3sink_mp3_parse_head(MP3Frame *frame, guint32 head)
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_mp3_frame_calc_tpf:
  *
  */
-
-void inline
+static void inline
 gst_dspmp3sink_mp3_frame_calc_tpf(MP3Frame *frame)
 {
   frame->tpf = bs[frame->layer];
@@ -552,7 +550,7 @@ gst_dspmp3sink_mp3_frame_calc_tpf(MP3Frame *frame)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_get_stream_details:
  * @dsp: DSP MP3 sink object
  * @quick_mode: Do not perform deep-scan for the stream. This
@@ -564,11 +562,9 @@ gst_dspmp3sink_mp3_frame_calc_tpf(MP3Frame *frame)
  *
  * Returns: TRUE if the info was found, otherwise FALSE
  */
-
 static gboolean
 gst_dspmp3sink_get_stream_details(GstDSPMP3Sink *dsp, gboolean quick_mode)
 {
-
   MP3Frame first_frame;
   MP3Frame tmp_frame;
   gint same_sr_cnt = 0, sr_old, sr_curr = -1;
@@ -668,15 +664,15 @@ gst_dspmp3sink_get_stream_details(GstDSPMP3Sink *dsp, gboolean quick_mode)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_setcaps:
  * @pad: GstPad that is the target for connect
  * @caps: GStCaps offered
  *
  * Returns: TRUE if caps are accepted
  */
-
-static gboolean gst_dspmp3sink_setcaps(GstBaseSink *sink, GstCaps *caps)
+static gboolean
+gst_dspmp3sink_setcaps(GstBaseSink *sink, GstCaps *caps)
 {
   GstDSPMP3Sink *dsp = GST_DSPMP3SINK (sink);
   gboolean ret = TRUE;
@@ -726,7 +722,7 @@ static gboolean gst_dspmp3sink_setcaps(GstBaseSink *sink, GstCaps *caps)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_calculate_seekpos:
  * @dsp: DSP MP3 sink object
  * @time_offset: the time where to seek
@@ -736,8 +732,8 @@ static gboolean gst_dspmp3sink_setcaps(GstBaseSink *sink, GstCaps *caps)
  *
  * Returns: seek position in bytes
  */
-
-gint64 gst_dspmp3sink_calculate_seekpos(GstDSPMP3Sink *dsp, guint64 time_offset)
+static gint64
+gst_dspmp3sink_calculate_seekpos(GstDSPMP3Sink *dsp, guint64 time_offset)
 {
    // XING-based seek
   if(dsp->xingtag.xframes && dsp->xingtag.xbytes && dsp->frame.tpf) {
@@ -784,14 +780,12 @@ gint64 gst_dspmp3sink_calculate_seekpos(GstDSPMP3Sink *dsp, guint64 time_offset)
 }
 
 
-
-/**
+/*
  * gst_dspmp3sink_perform_seek:
  * @dsp: DSP MP3 sink object
  *
  * Perform SEEK operation
  */
-
 static gboolean
 gst_dspmp3sink_perform_seek(GstDSPMP3Sink *dsp, guint64 time_offset)
 {
@@ -830,14 +824,16 @@ gst_dspmp3sink_perform_seek(GstDSPMP3Sink *dsp, guint64 time_offset)
 }
 
 
-/**
+/*
+ * gst_dspmp3sink_open:
+ *
  * Open the codec
  *
  */
-
-static gboolean gst_dspmp3sink_open(GstDSPMP3Sink *dsp, gint layer)
+static gboolean
+gst_dspmp3sink_open(GstDSPMP3Sink *dsp, gint layer)
 {
-  gchar *devname = dsp_node_names[layer]; // between 0-2
+  const gchar *devname = dsp_node_names[layer]; // between 0-2
 
   if(!gst_dspaudio_open(dsp->audio, devname)) {
     GST_ELEMENT_ERROR (dsp, RESOURCE,
@@ -853,7 +849,6 @@ static gboolean gst_dspmp3sink_open(GstDSPMP3Sink *dsp, gint layer)
  * Initialize the DSP for playback and send PLAY command
  *
  */
-
 static gboolean
 gst_dspmp3sink_loop_initialization(GstDSPMP3Sink *dsp, gboolean check_stream)
 {
@@ -894,11 +889,10 @@ gst_dspmp3sink_loop_initialization(GstDSPMP3Sink *dsp, gboolean check_stream)
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_fill_it:
  *
  */
-
 static gboolean
 gst_dspmp3sink_fill_it(GstDSPMP3Sink *dsp, int btr)
 {
@@ -923,13 +917,13 @@ gst_dspmp3sink_fill_it(GstDSPMP3Sink *dsp, int btr)
 }
 
 
-/**
- * Fill DSP's MMAP buffer with MP3 frames
+/*
+ * gst_dspmp3sink_copy_frame:
  *
+ * Fill DSP's MMAP buffer with MP3 frames
  *
  * Returns: TRUE if operation was successful
  */
-
 static gboolean
 gst_dspmp3sink_copy_frame(GstDSPMP3Sink *dsp)
 {
@@ -956,7 +950,6 @@ gst_dspmp3sink_copy_frame(GstDSPMP3Sink *dsp)
     }
     dsp->sync = TRUE;
 
-
     dsp->curr_fc++;
     dsp->frame_count++;
     dsp->total_frames++;
@@ -1002,9 +995,9 @@ gst_dspmp3sink_copy_frame(GstDSPMP3Sink *dsp)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_render:
- * sink:
+ * @sink:
  * @buffer:
  *
  * This method is automatically called by GStreamer framework, when
@@ -1014,7 +1007,8 @@ gst_dspmp3sink_copy_frame(GstDSPMP3Sink *dsp)
 
 #define STREAM_DETECT_THRESHOLD 10000
 
-static GstFlowReturn gst_dspmp3sink_render (GstBaseSink *sink, GstBuffer * buffer)
+static GstFlowReturn
+gst_dspmp3sink_render (GstBaseSink *sink, GstBuffer * buffer)
 {
   DSPWaitStatus status;
   gint ret;
@@ -1101,7 +1095,7 @@ static GstFlowReturn gst_dspmp3sink_render (GstBaseSink *sink, GstBuffer * buffe
 }
 
 
-/**
+/*
  * gst_dspmp3sink_initialize:
  * @dsp: DSP MP3 sink object
  *
@@ -1110,7 +1104,6 @@ static GstFlowReturn gst_dspmp3sink_render (GstBaseSink *sink, GstBuffer * buffe
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 static gboolean
 gst_dspmp3sink_initialize(GstDSPMP3Sink *dsp)
 {
@@ -1133,7 +1126,7 @@ gst_dspmp3sink_initialize(GstDSPMP3Sink *dsp)
 }
 
 
-/**
+/*
 * gst_dspmp3sink_preroll:
 * @pad:
 * @buffer:
@@ -1175,7 +1168,7 @@ gst_dspmp3sink_preroll (GstBaseSink *sink, GstBuffer * buffer)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_set_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -1184,7 +1177,6 @@ gst_dspmp3sink_preroll (GstBaseSink *sink, GstBuffer * buffer)
  *
  * Stores the property value into GObject
  */
-
 static void
 gst_dspmp3sink_set_property (GObject * object,
                              guint prop_id,
@@ -1199,7 +1191,7 @@ gst_dspmp3sink_set_property (GObject * object,
 }
 
 
-/**
+/*
  * gst_dspmp3sink_get_property:
  * @object: GObject where the property should be retrieved
  * @prop_id: Unique identifier of the desired property
@@ -1208,7 +1200,6 @@ gst_dspmp3sink_set_property (GObject * object,
  *
  * Retrieves the desired parameter value from object
  */
-
 static void
 gst_dspmp3sink_get_property (GObject * object,
                              guint prop_id,
@@ -1223,11 +1214,10 @@ gst_dspmp3sink_get_property (GObject * object,
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_get_query_types:
  *
  */
-
 static const GstQueryType *
 gst_dspmp3sink_get_query_types(GstElement * element)
 {
@@ -1242,7 +1232,7 @@ gst_dspmp3sink_get_query_types(GstElement * element)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_query:
  * @element: GstElement that received this query
  * @type: Type of the query, e.g. TOTAL, POSITION
@@ -1251,7 +1241,6 @@ gst_dspmp3sink_get_query_types(GstElement * element)
  *
  * Returns: TRUE, iff the query was successful.
  */
-
 static gboolean
 gst_dspmp3sink_query (GstElement * element, GstQuery *query)
 {
@@ -1356,72 +1345,73 @@ gst_dspmp3sink_query (GstElement * element, GstQuery *query)
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_send_remaining:
  *
  */
-
 static void
 gst_dspmp3sink_send_remaining(GstDSPMP3Sink *dsp)
 {
   DSPWaitStatus ret = DSP_WAIT_OK;
 
+  if (dsp->total_bytes == 0) {
+    DBG_PRINT ("No data sent - not need EOS\n");
+    return;
+  }
+
+  do {
+    g_mutex_lock(dsp->audio->dsp_mutex);
+    ret = gst_dspaudio_wait_buffer(dsp->audio);
+    g_mutex_unlock(dsp->audio->dsp_mutex);
+  }
+  while (ret == DSP_WAIT_RETRY);
+
+  if(ret != DSP_WAIT_OK) {
+    dsp->audio->mode = DSP_MODE_EOS;
+    return;
+  }
+
   if(gst_adapter_available(dsp->adapter)) {
-    do {
-      g_mutex_lock(dsp->audio->dsp_mutex);
-      ret = gst_dspaudio_wait_buffer(dsp->audio);
-      g_mutex_unlock(dsp->audio->dsp_mutex);
-    }
-    while (ret == DSP_WAIT_RETRY);
-
-    if(ret != DSP_WAIT_OK) {
-      return;
-    }
-
     while(gst_dspmp3sink_copy_frame(dsp)) {
       DBG_PRINT("EOF FRAME COPIED\n");
     }
-
-    if(dsp->mmap_ptr) {
-      short int tmp[2];
-      tmp[0] = DSP_CMD_EOF;
-      tmp[1] = dsp->mmap_ptr / sizeof(short int);
-
-      // Send remaining bytes to DSP
-      DBG_PRINT("WRITING EOF CMD to DSP (%d short ints)\n", tmp[1]);
-      g_mutex_lock(dsp->audio->dsp_mutex);
-      write(dsp->audio->codec.fd, tmp, 2*sizeof(short int));
-      dsp->audio->rw_pending = 0;
-      dsp->total_bytes += dsp->mmap_ptr;
-      dsp->audio->mode = DSP_MODE_EOS;
-      gst_dspaudio_wait_eos(dsp->audio);
-      g_mutex_unlock(dsp->audio->dsp_mutex);
-      DBG_PRINT("Wrote total of %lld bytes to DSP during playback\n", dsp->total_bytes);
-    }
   }
   else {
-    DBG_PRINT("NO DATA TO SEND -> NOT SENDING EOF\n");
+    DBG_PRINT("NO DATA TO SEND\n");
+    dsp->mmap_ptr = 0;
   }
+
+  short int tmp[2];
+  tmp[0] = DSP_CMD_EOF;
+  tmp[1] = dsp->mmap_ptr / sizeof(short int);
+
+  // Send remaining bytes to DSP
+  DBG_PRINT("WRITING EOF CMD to DSP (%d short ints)\n", tmp[1]);
+  g_mutex_lock(dsp->audio->dsp_mutex);
+  write(dsp->audio->codec.fd, tmp, 2*sizeof(short int));
+  dsp->audio->rw_pending = 0;
+  dsp->total_bytes += dsp->mmap_ptr;
+  dsp->audio->mode = DSP_MODE_EOS;
+  gst_dspaudio_wait_eos(dsp->audio);
+  g_mutex_unlock(dsp->audio->dsp_mutex);
+  DBG_PRINT("Wrote total of %lld bytes to DSP during playback\n", dsp->total_bytes);
 }
 
 
-/**
+/*
+ * gst_dspmp3sink_event2:
  *
  * This method handles events coming from basesink
  */
-
-static gboolean gst_dspmp3sink_event2 (GstBaseSink *sink, GstEvent * event)
+static gboolean
+gst_dspmp3sink_event2 (GstBaseSink *sink, GstEvent * event)
 {
   GstDSPMP3Sink *dsp = GST_DSPMP3SINK (sink);
 
   GstEventType type = event ? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;
   if(type == GST_EVENT_EOS) {
     DBG_PRINT("EOS EVENT\n");
-    dsp->eos_ongoing = TRUE;
-    dsp->audio->codec.stream_id = 0;
-    g_signal_emit (G_OBJECT (sink), gst_dspmp3sink_signals[DSPMP3_SIGNAL_STREAMID], 0, 0);
     gst_dspmp3sink_send_remaining(dsp);
-    dsp->eos_ongoing = FALSE;
   }
   else if(type == GST_EVENT_FLUSH_START) {
     g_mutex_lock(dsp->audio->dsp_mutex);
@@ -1440,15 +1430,15 @@ static gboolean gst_dspmp3sink_event2 (GstBaseSink *sink, GstEvent * event)
   return TRUE;
 }
 
-/**
+/*
  * gst_dspmp3sink_event:
  * @element: GstElement that received this event
  * @event: The GstEvent structure itself
  *
  * Returns: TRUE, iff the event was successful
  */
-
-static gboolean gst_dspmp3sink_event (GstElement * element, GstEvent * event)
+static gboolean
+gst_dspmp3sink_event (GstElement * element, GstEvent * event)
 {
   GstDSPMP3Sink *dsp = GST_DSPMP3SINK (element);
   gboolean res = FALSE;
@@ -1511,7 +1501,7 @@ static gboolean gst_dspmp3sink_event (GstElement * element, GstEvent * event)
 }
 
 
-/**
+/*
  * gst_dspmp3sink_change_state:
  * @element: The GstElement whose state to be changed
  *
@@ -1522,15 +1512,12 @@ static gboolean gst_dspmp3sink_event (GstElement * element, GstEvent * event)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 static GstStateChangeReturn
 gst_dspmp3sink_change_state (GstElement * element, GstStateChange transition)
 {
   GstDSPMP3Sink *dspmp3sink = GST_DSPMP3SINK (element);
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
-  gst_dspaudio_clock_change_state(dspmp3sink->audio, transition);
-
   switch (transition) {
 
     case GST_STATE_CHANGE_NULL_TO_READY:
@@ -1542,7 +1529,6 @@ gst_dspmp3sink_change_state (GstElement * element, GstStateChange transition)
       dspmp3sink->xingtag.xframes = 0;
       dspmp3sink->frame_seek_base = 0;
       dspmp3sink->outbuf.dsp_cmd = DSP_CMD_DATA_WRITE;
-      dspmp3sink->eos_ongoing = FALSE;
       char_in_tmp = FALSE;
       bzero(&dspmp3sink->frame, sizeof(MP3Frame));
       break;
@@ -1556,8 +1542,9 @@ gst_dspmp3sink_change_state (GstElement * element, GstStateChange transition)
       DBG_PRINT("MP3 PAUSED TO PLAYING\n");
       g_mutex_lock(dspmp3sink->audio->dsp_mutex);
 
-      if(dspmp3sink->audio->mode == DSP_MODE_PAUSED) {
-        // Set the play only if we are at paused state
+      if(dspmp3sink->audio->mode == DSP_MODE_PAUSED ||
+         dspmp3sink->audio->mode == DSP_MODE_EOS) {
+        // Set the play only if we are at paused or EOS state
         gst_dspaudio_play(dspmp3sink->audio);
         dspmp3sink->audio->rw_pending = 0;
       }
@@ -1574,6 +1561,7 @@ gst_dspmp3sink_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
+  gst_dspaudio_clock_change_state(dspmp3sink->audio, transition);
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch(transition) {
@@ -1581,20 +1569,22 @@ gst_dspmp3sink_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       DBG_PRINT("MP3 PLAYING TO PAUSED V2\n");
       gst_dspaudio_remove_interrupt(dspmp3sink->audio);
-      if(dspmp3sink->audio->mode != DSP_MODE_EOS) {
-        g_mutex_lock(dspmp3sink->audio->dsp_mutex);
-        if(gst_dspaudio_pause(dspmp3sink->audio)) {
-          gst_dspaudio_read_cmd(dspmp3sink->audio, 0);
-          DBG_PRINT("PAUSE command sent to DSP\n");
-          dspmp3sink->audio->rw_pending = 0;
-        }
-        g_mutex_unlock(dspmp3sink->audio->dsp_mutex);
+      g_mutex_lock(dspmp3sink->audio->dsp_mutex);
+      DBG_PRINT("Got the mutex\n");
+      if(gst_dspaudio_pause(dspmp3sink->audio)) {
+        gst_dspaudio_read_cmd(dspmp3sink->audio, 0);
+        DBG_PRINT("PAUSE command sent to DSP\n");
+        dspmp3sink->audio->rw_pending = 0;
       }
+      g_mutex_unlock(dspmp3sink->audio->dsp_mutex);
       DBG_PRINT("MP3 PLAYING TO PAUSED V2 OK\n");
       break;
 
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       DBG_PRINT("MP3 PAUSED TO READY\n");
+      dspmp3sink->audio->codec.stream_id = 0;
+      g_signal_emit (G_OBJECT (dspmp3sink),
+                     gst_dspmp3sink_signals[DSPMP3_SIGNAL_STREAMID], 0, 0);
       if(gst_dspaudio_stop(dspmp3sink->audio)) {
         gst_dspaudio_read_cmd(dspmp3sink->audio, 0);
       }
@@ -1617,11 +1607,10 @@ gst_dspmp3sink_change_state (GstElement * element, GstStateChange transition)
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_async_play:
  *
  */
-
 static GstStateChangeReturn
 gst_dspmp3sink_async_play (GstBaseSink *sink)
 {
@@ -1638,30 +1627,24 @@ gst_dspmp3sink_async_play (GstBaseSink *sink)
 }
 
 
-/**
- *
+/*
+ * gst_dspmp3sink_provide_clock:
  *
  */
-
 static GstClock *
 gst_dspmp3sink_provide_clock (GstElement * elem)
 {
   DBG_PRINT("gst_dspmp3sink_provide_clock\n");
   GstDSPMP3Sink *dsp = GST_DSPMP3SINK (elem);
-  return GST_CLOCK_CAST (gst_object_ref (dsp->audio->clock));
-//   return GST_CLOCK (gst_object_ref (GST_OBJECT (dsp->audio->clock)));
+  return GST_CLOCK_CAST (gst_object_ref (dsp->clock));
 }
 
 
-/**
- *
- *
- */
-
 /*
+ * gst_dspmp3sink_set_clock:
+ *
  * For testing
  *
-
 static gboolean
 gst_dspmp3sink_set_clock(GstElement *element, GstClock *clock)
 {
@@ -1675,11 +1658,11 @@ gst_dspmp3sink_set_clock(GstElement *element, GstClock *clock)
 }
 */
 
-/**
- *
+
+/*
+ * gst_dspmp3sink_get_time:
  *
  */
-
 static GstClockTime
 gst_dspmp3sink_get_time (GstClock * clock, gpointer sink)
 {
@@ -1688,7 +1671,7 @@ gst_dspmp3sink_get_time (GstClock * clock, gpointer sink)
 }
 
 
-/**
+/*
  * plugin_init:
  * @plugin: GstPlugin to be initialized
  *
@@ -1696,7 +1679,6 @@ gst_dspmp3sink_get_time (GstClock * clock, gpointer sink)
  *
  * Returns: Boolean value, TRUE if initialization was successful
  */
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -1704,12 +1686,6 @@ plugin_init (GstPlugin * plugin)
       GST_TYPE_DSPMP3SINK);
 }
 
-
-/**
- * This is the "boilerplate" of the plugin
- *
- */
-
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
                    GST_VERSION_MINOR,
                    "dspmp3",
@@ -1718,4 +1694,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
                    VERSION,
                    GST_LICENSE_UNKNOWN,
                    "dspaudio",
-                   "Nokia Corporation");
+                   "http://www.nokia.com/");
