@@ -29,43 +29,39 @@
 
 #include "gstdsppcmsink.h"
 
-GstElementDetails gst_dsppcmsink_details = GST_ELEMENT_DETAILS ("DSP PCM Sink",
-    "Sink/Audio",
-    "Raw DSP PCM audio sink",
-    "Makoto Sugano <makoto.sugano@nokia.com>");
 
-static gchar *pcm_sink_device ="/dev/dsptask/pcm2";
+static const gchar *pcm_sink_device ="/dev/dsptask/pcm2";
 
 
 static GstStaticPadTemplate sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-int, "
-        "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", "
-        "signed = (boolean) { TRUE, FALSE }, "
-        "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, "
-        "rate = (int) {  8000, 11025, 12000, \
-                        16000, 22050, 24000, \
-                        32000, 44100, 48000, \
-                        64000, 88200, 96000 }, "
-        "channels = (int) [ 1, 2 ]; "
+        GST_PAD_SINK,
+        GST_PAD_ALWAYS,
+        GST_STATIC_CAPS ("audio/x-raw-int, "
+            "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", "
+            "signed = (boolean) { TRUE, FALSE }, "
+            "width = (int) { 8, 16 }, "
+            "depth = (int) { 8, 16 }, "
+            "rate = (int) {  8000, 11025, 12000, "
+                          " 16000, 22050, 24000, "
+                          " 32000, 44100, 48000, "
+                          "  64000, 88200, 96000 }, "
+            "channels = (int) [ 1, 2 ]; "
 
-        "audio/x-alaw, "
-        "rate = (int) {  8000, 11025, 12000, \
-                        16000, 22050, 24000, \
-                        32000, 44100, 48000, \
-                        64000, 88200, 96000 }, "
-        "channels = (int) [ 1, 2 ]; "
+            "audio/x-alaw, "
+            "rate = (int) {  8000, 11025, 12000, "
+                          " 16000, 22050, 24000, "
+                          " 32000, 44100, 48000, "
+                          " 64000, 88200, 96000 }, "
+            "channels = (int) [ 1, 2 ]; "
 
-        "audio/x-mulaw, "
-        "rate = (int) {  8000, 11025, 12000, \
-                        16000, 22050, 24000, \
-                        32000, 44100, 48000, \
-                        64000, 88200, 96000 }, "
-        "channels = (int) [ 1, 2 ]"
-      )
+            "audio/x-mulaw, "
+            "rate = (int) {  8000, 11025, 12000, "
+                          " 16000, 22050, 24000, "
+                          " 32000, 44100, 48000, "
+                          " 64000, 88200, 96000 }, "
+            "channels = (int) [ 1, 2 ]"
+         )
     );
 
 
@@ -91,19 +87,18 @@ static gboolean gst_dsppcmsink_event2 (GstBaseSink *sink, GstEvent *event);
 static GstClock *gst_dsppcmsink_provide_clock (GstElement * elem);
 static GstClockTime gst_dsppcmsink_get_time (GstClock * clock, gpointer sink);
 
-/**
- *
- *
- */
-
-void _do_init()
+#ifdef DEBUG
+static void _do_init()
 {
   DBG_PRINT("PCMSINK DO_INIT\n");
 }
 
 GST_BOILERPLATE_FULL (GstDSPPcmSink, gst_dsppcmsink, GstBaseSink,
                       GST_TYPE_BASE_SINK, _do_init);
-
+#else
+GST_BOILERPLATE (GstDSPPcmSink, gst_dsppcmsink, GstBaseSink,
+                 GST_TYPE_BASE_SINK);
+#endif
 
 
 #define GST_TYPE_DSPPCMSINK_AEC_MODE (gst_dsppcmsink_aec_mode_get_type())
@@ -117,7 +112,7 @@ gst_dsppcmsink_aec_mode_get_type (void)
     {0, NULL, NULL},
   };
 
-  if (!gst_dsppcmsink_aec_mode_type) {
+  if (G_UNLIKELY(!gst_dsppcmsink_aec_mode_type)) {
     gst_dsppcmsink_aec_mode_type = g_enum_register_static ("GstPCMSinkAECMode",
         dsppcmsink_aec_modes);
   }
@@ -131,15 +126,15 @@ gst_dsppcmsink_aec_mode_get_type (void)
  * Deletes PCM sink element instance. Called automatically by
  * GLib framework when element needs to be disposed.
  */
-
 static void
 gst_dsppcmsink_dispose (GObject * object)
 {
   DBG_PRINT("gst_dsppcmsink_dispose\n");
   GstDSPPcmSink *dsp = (GstDSPPcmSink *) object;
 
-  if(dsp->audio->clock) {
-    gst_object_unref(dsp->audio->clock);
+  if(dsp->clock) {
+    gst_object_unref(dsp->clock);
+    dsp->clock = NULL;
   }
 
   if(dsp->audio) {
@@ -159,16 +154,19 @@ gst_dsppcmsink_dispose (GObject * object)
  * Does the basic initialization of PCM sink element class. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dsppcmsink_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  static const GstElementDetails details = GST_ELEMENT_DETAILS (
+    "DSP PCM Sink",
+    "Sink/Audio",
+    "Raw DSP PCM audio sink",
+    "Makoto Sugano <makoto.sugano@nokia.com>");
 
   gst_element_class_add_pad_template (gstelement_class,
       gst_static_pad_template_get (&sink_template));
-
-  gst_element_class_set_details (gstelement_class, &gst_dsppcmsink_details);
+  gst_element_class_set_details (gstelement_class, &details);
 
 }
 
@@ -180,7 +178,6 @@ gst_dsppcmsink_base_init (gpointer g_class)
  * Initializes PCM sink element class. This is called by GStreamer
  * framework.
  */
-
 static void
 gst_dsppcmsink_class_init (GstDSPPcmSinkClass * klass)
 {
@@ -230,16 +227,16 @@ gst_dsppcmsink_class_init (GstDSPPcmSinkClass * klass)
  * resources. This also creates an audio stream object. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dsppcmsink_init (GstDSPPcmSink * dsp, GstDSPPcmSinkClass *g_class)
 {
   // This is why we need dispose()
   dsp->audio = gst_dspaudio_new();
+  gst_base_sink_set_sync (GST_BASE_SINK (dsp), FALSE);
 
-  dsp->audio->clock = gst_audio_clock_new ("clock",
-                                           (GstAudioClockGetTimeFunc) gst_dsppcmsink_get_time,
-                                           dsp);
+  dsp->clock = gst_audio_clock_new ("clock",
+                                    (GstAudioClockGetTimeFunc) gst_dsppcmsink_get_time,
+                                    dsp);
 }
 
 
@@ -257,7 +254,6 @@ gst_dsppcmsink_init (GstDSPPcmSink * dsp, GstDSPPcmSinkClass *g_class)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 static gboolean
 gst_dsppcmsink_format_get (gint law, gint endianness, gboolean sign, gint width,
     gint depth, guint * format, gint * bps)
@@ -319,7 +315,6 @@ gst_dsppcmsink_format_get (gint law, gint endianness, gboolean sign, gint width,
  *
  *
  */
-
 static GstCaps *
 gst_dsppcmsink_getcaps (GstBaseSink *sink)
 {
@@ -332,9 +327,8 @@ gst_dsppcmsink_getcaps (GstBaseSink *sink)
  *
  *
  */
-
 static gboolean
-    gst_dsppcmsink_initialize(GstDSPPcmSink *dsp)
+gst_dsppcmsink_initialize(GstDSPPcmSink *dsp)
 {
   // Read the junk out...
   gst_dspaudio_flush(&dsp->audio->codec);
@@ -400,7 +394,6 @@ static gboolean
  *
  *
  */
-
 static gboolean
 gst_dsppcmsink_setcaps (GstBaseSink *src, GstCaps *caps)
 {
@@ -472,7 +465,6 @@ gst_dsppcmsink_setcaps (GstBaseSink *src, GstCaps *caps)
  *
  *
  */
-
 static GstFlowReturn
 gst_dsppcmsink_render (GstBaseSink *base, GstBuffer * buffer)
 {
@@ -482,6 +474,14 @@ gst_dsppcmsink_render (GstBaseSink *base, GstBuffer * buffer)
   guint length = GST_BUFFER_SIZE(buffer);
   guint dataIndex = 0, tw, tw1;
 
+  if(dsp->audio->mode == DSP_MODE_CONFIGURED) {
+    g_mutex_lock(dsp->audio->dsp_mutex);
+    gst_dspaudio_flush(&dsp->audio->codec);
+    gst_dspaudio_play(dsp->audio);
+    gst_dspaudio_update_dsp_settings(dsp->audio);
+    g_mutex_unlock(dsp->audio->dsp_mutex);
+  }
+
   while ( length > 0 ) {
 
     if(dsp->audio->mode == DSP_MODE_ERROR) {
@@ -494,16 +494,8 @@ gst_dsppcmsink_render (GstBaseSink *base, GstBuffer * buffer)
 
       return GST_FLOW_ERROR;
     }
-    else if(dsp->audio->mode == DSP_MODE_CONFIGURED) {
-      g_mutex_lock(dsp->audio->dsp_mutex);
-      gst_dspaudio_flush(&dsp->audio->codec);
-      gst_dspaudio_play(dsp->audio);
-      gst_dspaudio_update_dsp_settings(dsp->audio);
-      g_mutex_unlock(dsp->audio->dsp_mutex);
-    }
 
     g_mutex_lock(dsp->audio->dsp_mutex);
-
     status = gst_dspaudio_wait_buffer(dsp->audio);
 
     if(status == DSP_WAIT_RETRY) {
@@ -559,7 +551,6 @@ gst_dsppcmsink_render (GstBaseSink *base, GstBuffer * buffer)
  *
  * Stores the property value into GObject
  */
-
 static void
 gst_dsppcmsink_set_property (GObject * object, guint prop_id, const GValue * value,
     GParamSpec * pspec)
@@ -585,7 +576,6 @@ gst_dsppcmsink_set_property (GObject * object, guint prop_id, const GValue * val
  *
  * Retrieves the desired parameter value from object
  */
-
 static void
 gst_dsppcmsink_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
@@ -606,8 +596,8 @@ gst_dsppcmsink_get_property (GObject * object, guint prop_id, GValue * value,
  *
  *
  */
-
-static gboolean gst_dsppcmsink_event2(GstBaseSink *sink, GstEvent *event)
+static gboolean
+gst_dsppcmsink_event2(GstBaseSink *sink, GstEvent *event)
 {
   DBG_PRINT("PCM SINK EVENT\n");
   GstDSPPcmSink *dsp = GST_DSPPCMSINK (sink);
@@ -674,17 +664,14 @@ static gboolean gst_dsppcmsink_event2(GstBaseSink *sink, GstEvent *event)
  *
  * Returns: TRUE if operation was successful, otherwise FALSE
  */
-
 static GstStateChangeReturn
 gst_dsppcmsink_change_state(GstElement * element, GstStateChange transition)
 {
   GstDSPPcmSink *dsp = GST_DSPPCMSINK (element);
-  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS, tmp;
+  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   DBG_PRINT("PCM2 TRANSITION: %d\n", transition);
 
-  gst_dspaudio_clock_change_state(dsp->audio, transition);
-
   switch (transition) {
 
     case GST_STATE_CHANGE_NULL_TO_READY:
@@ -724,11 +711,9 @@ gst_dsppcmsink_change_state(GstElement * element, GstStateChange transition)
       break;
   }
 
-  tmp = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-  if(tmp != GST_STATE_CHANGE_SUCCESS) {
-    DBG_PRINT("PCM PARENT CLASS STATE CHANGE: %d\n", tmp);
-    ret = tmp;
-  }
+  gst_dspaudio_clock_change_state(dsp->audio, transition);
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  DBG_PRINT("PCM PARENT CLASS STATE CHANGE: %d\n", ret);
 
   switch (transition) {
 
@@ -779,13 +764,12 @@ gst_dsppcmsink_change_state(GstElement * element, GstStateChange transition)
  *
  *
  */
-
 static GstClock *
 gst_dsppcmsink_provide_clock (GstElement * elem)
 {
   DBG_PRINT("gst_dsppcmsink_provide_clock\n");
   GstDSPPcmSink *dsp = GST_DSPPCMSINK (elem);
-  return GST_CLOCK (gst_object_ref (GST_OBJECT (dsp->audio->clock)));
+  return GST_CLOCK (gst_object_ref (GST_OBJECT (dsp->clock)));
 }
 
 
@@ -793,7 +777,6 @@ gst_dsppcmsink_provide_clock (GstElement * elem)
  *
  *
  */
-
 static GstClockTime
 gst_dsppcmsink_get_time (GstClock * clock, gpointer sink)
 {
