@@ -28,12 +28,9 @@
 
 #include "gstdsppcmsrc.h"
 
-GstElementDetails gst_dsppcmsrc_details = GST_ELEMENT_DETAILS ("DSP PCM Src",
-    "Source/Audio",
-    "PCM audio src",
-    "Makoto Sugano <makoto.sugano@nokia.com>");
+#define DSP_PCM_SID_UPDATE_FRAMESIZE 22
 
-static gchar *dsp_node_names[2] =
+static const gchar *dsp_node_names[2] =
 {
   "/dev/dsptask/pcm_rec",
   "/dev/dsptask/g711_enc"
@@ -42,24 +39,34 @@ static gchar *dsp_node_names[2] =
 
 static GstStaticPadTemplate dsppcmsrc_src_template =
     GST_STATIC_PAD_TEMPLATE ("src",
-                             GST_PAD_SRC,
-                             GST_PAD_ALWAYS,
-                             GST_STATIC_CAPS ( "audio/x-raw-int, "
-                                 "endianness = (int) { " G_STRINGIFY (G_BYTE_ORDER) " }, "
-                                 "signed = (boolean) TRUE, "
-                                 "width = (int) 16, "
-                                 "depth = (int) 16, "
-                                 "rate = (int) 8000, "
-                                 "channels = (int) 1;"
+       GST_PAD_SRC,
+       GST_PAD_ALWAYS,
+       GST_STATIC_CAPS ( "audio/x-raw-int, "
+           "endianness = (int) { " G_STRINGIFY (G_BYTE_ORDER) " }, "
+           "signed = (boolean) TRUE, "
+           "width = (int) 16, "
+           "depth = (int) 16, "
+           "rate = (int) 8000, "
+           "channels = (int) 1;"
 
-                                 "audio/x-alaw, "
-                                 "rate = (int) 8000, "
-                                 "channels = (int) 1; "
+           "audio/x-alaw, "
+           "rate = (int) 8000, "
+           "channels = (int) 1; "
 
-                                 "audio/x-mulaw, "
-                                 "rate = (int) 8000, "
-                                 "channels = (int) 1"
-                              ) );
+           "audio/x-mulaw, "
+           "rate = (int) 8000, "
+           "channels = (int) 1"
+        )
+     );
+
+static GstStaticPadTemplate dsppcmsrc_cnsrc_template =
+GST_STATIC_PAD_TEMPLATE ("cnsrc%d",
+    GST_PAD_SRC,
+    GST_PAD_REQUEST,
+    GST_STATIC_CAPS ( "audio/CN, "
+                      "rate = (int) 8000, "
+                      "channels = (int) 1"
+                    ) );
 
 // Function prototypes
 
@@ -78,42 +85,31 @@ static gboolean gst_dsppcmsrc_setcaps (GstBaseSrc * src, GstCaps *caps);
 static GstFlowReturn gst_dsppcmsrc_create (GstPushSrc * psrc, GstBuffer ** buf);
 static gboolean gst_dsppcmsrc_start (GstBaseSrc * bsrc);
 static gboolean gst_dsppcmsrc_stop (GstBaseSrc * bsrc);
+static gboolean gst_dsppcmsrc_event (GstBaseSrc *src, GstEvent *event);
 
 static GstStateChangeReturn
-    gst_dsppcmsrc_change_state (GstElement * element, GstStateChange transition);
+gst_dsppcmsrc_change_state (GstElement * element, GstStateChange transition);
 
-/**
- *
- *
- */
+static GstPad * gst_dsppcmsrc_request_new_pad (GstElement     *element,
+                                               GstPadTemplate *templ,
+                                               const gchar    *name);
 
-void _pcmsrc_do_init()
+static void
+gst_dsppcmsrc_release_pad (GstElement *element, GstPad *pad);
+
+
+#ifdef DEBUG
+static void _do_init()
 {
   DBG_PRINT("PCM DO_INIT\n");
 }
 
 GST_BOILERPLATE_FULL (GstDSPPCMSrc, gst_dsppcmsrc, GstPushSrc,
-                      GST_TYPE_PUSH_SRC, _pcmsrc_do_init);
-
-
-
-#define GST_TYPE_PCMSRC_DTX_MODE (gst_dsppcmsrc_dtx_mode_get_type())
-static GType
-gst_dsppcmsrc_dtx_mode_get_type (void)
-{
-  static GType dsppcmsrc_dtx_mode_type = 0;
-  static GEnumValue dsppcmsrc_dtx_modes[] = {
-    {GST_PCMSRC_DTX_MODE_OFF, "Don't use DTX", "off"},
-    {GST_PCMSRC_DTX_MODE_ON, "Use DTX", "on"},
-    {0, NULL, NULL},
-  };
-
-  if (!dsppcmsrc_dtx_mode_type) {
-    dsppcmsrc_dtx_mode_type = g_enum_register_static ("GstPCMSrcDTXMode",
-        dsppcmsrc_dtx_modes);
-  }
-  return dsppcmsrc_dtx_mode_type;
-}
+                      GST_TYPE_PUSH_SRC, _do_init);
+#else
+GST_BOILERPLATE (GstDSPPCMSrc, gst_dsppcmsrc, GstPushSrc,
+                 GST_TYPE_PUSH_SRC);
+#endif
 
 
 /**
@@ -123,17 +119,26 @@ gst_dsppcmsrc_dtx_mode_get_type (void)
  * Deletes PCM src element instance. Called automatically by
  * GLib framework when element needs to be disposed.
  */
-
 static void
 gst_dsppcmsrc_dispose (GObject * object)
 {
   GstDSPPCMSrc *dsp = (GstDSPPCMSrc *) object;
 
+  if(dsp->clock) {
+    gst_object_unref(dsp->clock);
+    dsp->clock = NULL;
+  }
+
   if(dsp->audio) {
     gst_dspaudio_destroy(dsp->audio);
     dsp->audio = NULL;
   }
 
+  if (dsp->lastcn) {
+    gst_buffer_unref (dsp->lastcn);
+    dsp->lastcn = NULL;
+  }
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -145,15 +150,21 @@ gst_dsppcmsrc_dispose (GObject * object)
  * Does the basic initialization of PCM src element class. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dsppcmsrc_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  static const GstElementDetails details = GST_ELEMENT_DETAILS (
+    "DSP PCM Src",
+    "Source/Audio",
+    "PCM audio src",
+    "Makoto Sugano <makoto.sugano@nokia.com>");
 
   gst_element_class_add_pad_template (gstelement_class,
-                                      gst_static_pad_template_get (&dsppcmsrc_src_template));
-  gst_element_class_set_details (gstelement_class, &gst_dsppcmsrc_details);
+      gst_static_pad_template_get (&dsppcmsrc_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&dsppcmsrc_cnsrc_template));
+  gst_element_class_set_details (gstelement_class, &details);
 
 }
 
@@ -164,7 +175,6 @@ gst_dsppcmsrc_base_init (gpointer g_class)
  * Initializes PCM src element class. This is called by GStreamer
  * framework.
  */
-
 static void
 gst_dsppcmsrc_class_init (GstDSPPCMSrcClass * klass)
 {
@@ -190,16 +200,19 @@ gst_dsppcmsrc_class_init (GstDSPPCMSrcClass * klass)
   g_object_class_install_property (gobject_class, DSPPCM_PROP_DTX_MODE,
                                    g_param_spec_enum ("dtx", "DTX mode",
                                        "Discontinuous transmission mode (G.711 only)",
-                                       GST_TYPE_PCMSRC_DTX_MODE,        /* enum type */
-                                       GST_PCMSRC_DTX_MODE_OFF,         /* default value */
+                                       GST_TYPE_DSPAUDIO_DTX_MODE, /* enum type */
+                                       GST_DSPAUDIO_DTX_MODE_OFF,  /* default value */
                                        G_PARAM_READWRITE));
 
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_dsppcmsrc_change_state);
+  gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR(gst_dsppcmsrc_request_new_pad);
+  gstelement_class->release_pad = GST_DEBUG_FUNCPTR(gst_dsppcmsrc_release_pad);
 
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_dsppcmsrc_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_dsppcmsrc_stop);
   gstbasesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_dsppcmsrc_getcaps);
   gstbasesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_dsppcmsrc_setcaps);
+  gstbasesrc_class->event = GST_DEBUG_FUNCPTR (gst_dsppcmsrc_event);
 
   gstpushsrc_class->create = GST_DEBUG_FUNCPTR (gst_dsppcmsrc_create);
 }
@@ -213,15 +226,14 @@ gst_dsppcmsrc_class_init (GstDSPPCMSrcClass * klass)
  * resources. This also creates an audio stream object. This is
  * called by GStreamer framework.
  */
-
 static void
 gst_dsppcmsrc_init (GstDSPPCMSrc * dsppcmsrc, GstDSPPCMSrcClass *g_class)
 {
   dsppcmsrc->audio = gst_dspaudio_new();
-  GST_BASE_SRC(dsppcmsrc)->blocksize = 80; // Default size = 10ms frame
-  dsppcmsrc->dtxmode = GST_PCMSRC_DTX_MODE_ON;
+  gst_base_src_set_live (GST_BASE_SRC (dsppcmsrc), TRUE);
+  dsppcmsrc->dtxmode = GST_DSPAUDIO_DTX_MODE_OFF;
+  GST_BASE_SRC (dsppcmsrc)->blocksize = 160;    // Default value
 
-  // FIXME: Do we need to delete this?
   dsppcmsrc->clock = gst_audio_clock_new ("clock",
                                           (GstAudioClockGetTimeFunc) gst_dsppcmsrc_get_time,
                                           dsppcmsrc);
@@ -233,7 +245,6 @@ gst_dsppcmsrc_init (GstDSPPCMSrc * dsppcmsrc, GstDSPPCMSrcClass *g_class)
  *
  *
  */
-
 static GstCaps *
 gst_dsppcmsrc_getcaps (GstBaseSrc * src)
 {
@@ -254,7 +265,6 @@ gst_dsppcmsrc_getcaps (GstBaseSrc * src)
  * Rounds the blocksize to be dividable by 80 (that is 10ms in time)
  * and makes sure that it will be between certain limits
  */
-
 static guint
 gst_dsppcmsrc_round_bsize(guint bsize, guint min, guint max)
 {
@@ -273,7 +283,6 @@ gst_dsppcmsrc_round_bsize(guint bsize, guint min, guint max)
  *
  * Returns: TRUE if caps can be accepted, otherwise FALSE
  */
-
 static gboolean
 gst_dsppcmsrc_setcaps(GstBaseSrc *bsrc, GstCaps * caps)
 {
@@ -291,10 +300,11 @@ gst_dsppcmsrc_setcaps(GstBaseSrc *bsrc, GstCaps * caps)
     init_data.frame_size = gst_dsppcmsrc_round_bsize(bsrc->blocksize, 80, 800);
     dsp->framesize = init_data.frame_size * sizeof(short int);
     dsp->devindex = 0;
+    dsp->raw = TRUE;
   }
   else if(!strncmp(mimetype, "audio/x-alaw", 12)) {
     DBG_PRINT("a-law\n");
-    if(dsp->dtxmode == GST_PCMSRC_DTX_MODE_ON) {
+    if(dsp->dtxmode == GST_DSPAUDIO_DTX_MODE_ON) {
       init_data.audio_fmt = DSP_AFMT_ALAW_DTX;
       init_data.frame_size = gst_dsppcmsrc_round_bsize(bsrc->blocksize, 160, 800);
       dsp->dtx_enabled = TRUE;
@@ -307,10 +317,11 @@ gst_dsppcmsrc_setcaps(GstBaseSrc *bsrc, GstCaps * caps)
     }
     dsp->framesize = init_data.frame_size;
     dsp->devindex = 1;
+    dsp->raw = FALSE;
   }
   else if(!strncmp(mimetype, "audio/x-mulaw", 12)) {
     DBG_PRINT("mu-law\n");
-    if(dsp->dtxmode == GST_PCMSRC_DTX_MODE_ON) {
+    if(dsp->dtxmode == GST_DSPAUDIO_DTX_MODE_ON) {
       init_data.audio_fmt = DSP_AFMT_ULAW_DTX;
       init_data.frame_size = gst_dsppcmsrc_round_bsize(bsrc->blocksize, 160, 800);
       dsp->dtx_enabled = TRUE;
@@ -323,6 +334,7 @@ gst_dsppcmsrc_setcaps(GstBaseSrc *bsrc, GstCaps * caps)
     }
     dsp->framesize = init_data.frame_size;
     dsp->devindex = 1;
+    dsp->raw = FALSE;
   }
   else {
     DBG_PRINT("Unknown format\n");
@@ -368,12 +380,10 @@ gst_dsppcmsrc_setcaps(GstBaseSrc *bsrc, GstCaps * caps)
  * @buffer:
  *
  */
-
 static GstFlowReturn
 gst_dsppcmsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
 {
   GstDSPPCMSrc *dsp = GST_DSPPCMSRC(psrc);
-  guint recv_frames = 1;
 
   if(dsp->audio->mode == DSP_MODE_ERROR) {
     GST_ELEMENT_ERROR (dsp, RESOURCE, READ, (NULL),
@@ -383,22 +393,108 @@ gst_dsppcmsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
     return GST_FLOW_ERROR;
   }
 
+  if (dsp->raw == FALSE) {
+    for (;;) {
+      *buf = gst_dspaudio_read_frame(dsp->audio, dsp->framesize,
+                                     DSP_PCM_SID_UPDATE_FRAMESIZE, TRUE);
 
-  if(dsp->dtx_enabled) {
-    *buf = gst_dspaudio_read_dtx_frame(dsp->audio, dsp->framesize);
+      if (!*buf)
+        return GST_FLOW_WRONG_STATE;
+
+      GST_BUFFER_DURATION(*buf) = dsp->frametimemillis * GST_MSECOND;
+      GST_BUFFER_TIMESTAMP(*buf) = dsp->framecount * dsp->frametimemillis * GST_MSECOND;
+      dsp->framecount++;
+
+      DBG_PRINT("G.711 src created a buffer\n");
+
+        /* SID_FRAME_NO_UPDATE: try to resend old CN frame */
+      if (GST_BUFFER_SIZE(*buf) == 0) {
+        gst_buffer_unref(*buf);
+        if (dsp->lastcn && dsp->cnpad) {
+          gst_buffer_ref(dsp->lastcn);
+          *buf = dsp->lastcn;
+        } else {
+          continue;
+        }
+      } else if (GST_BUFFER_FLAG_IS_SET(*buf, GST_BUFFER_FLAG_LAST)) {
+        /* is SID_FRAME_UPDATE */
+        if (!dsp->cnpad) {
+          gst_buffer_unref(*buf);
+          continue;
+        }
+        /* Save for future NO_UPDATE */
+        if (dsp->lastcn)
+          gst_buffer_unref(dsp->lastcn);
+
+        gst_buffer_ref(*buf);
+        dsp->lastcn = *buf;
+        gst_buffer_set_caps (dsp->lastcn, GST_PAD_CAPS (dsp->cnpad));
+      }
+
+      if (GST_BUFFER_CAPS(*buf)) {
+        /* CN Buffer */
+        GST_BUFFER_FLAG_UNSET(*buf, GST_BUFFER_FLAG_LAST);
+        gst_pad_push(dsp->cnpad, *buf);
+        continue;
+      } else {
+      /* Regular buffer */
+        gst_buffer_set_caps (*buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
+        return GST_FLOW_OK;
+      }
+    }
   }
   else {
-    *buf = gst_dspaudio_read_normal_frame(dsp->audio, dsp->framesize, &recv_frames);
-  }
+    // We need to read the frame by ourselves, since raw-PCM mode does not
+    // send DTX information
+    short int tmp = DSP_CMD_DATA_READ;
+    DSPWaitStatus status;
+    guint recv_frames = 1;
 
-  if(*buf) {
-    GST_BUFFER_DURATION(*buf) = recv_frames * dsp->frametimemillis * GST_MSECOND;
-    GST_BUFFER_TIMESTAMP(*buf) = dsp->framecount * dsp->frametimemillis * GST_MSECOND;
-    gst_buffer_set_caps (*buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
-    dsp->framecount += recv_frames;
-    return GST_FLOW_OK;
+    do {
+        g_mutex_lock (dsp->audio->dsp_mutex);
+        status = gst_dspaudio_wait_buffer (dsp->audio);
+        if (status == DSP_WAIT_RETRY) {
+            g_mutex_unlock (dsp->audio->dsp_mutex);
+            continue;
+        }
+        if (status == DSP_WAIT_INTERRUPT) {
+            g_mutex_unlock (dsp->audio->dsp_mutex);
+            break;
+        }
+        else if (status == DSP_WAIT_ERROR) {
+            g_mutex_unlock (dsp->audio->dsp_mutex);
+            dsp->audio->mode = DSP_MODE_ERROR;
+            break;
+        }
+
+        recv_frames = dsp->audio->readinfo.frame_size /
+                        (dsp->framesize / sizeof (short int));
+
+        *buf = gst_buffer_new_and_alloc (recv_frames * dsp->framesize);
+        guint8 *data = GST_BUFFER_DATA (*buf);
+
+        DBG_PRINT ("Received %d frames (framesize = %d short ints)",
+                    recv_frames, dsp->audio->readinfo.frame_size);
+
+        memcpy (data, dsp->audio->codec.mmap_buffer,
+                dsp->audio->readinfo.frame_size * sizeof (short int));
+
+        if (write (dsp->audio->codec.fd, &tmp, sizeof (short int)) < 0) {
+            dsp->audio->mode = DSP_MODE_ERROR;
+        }
+        g_mutex_unlock (dsp->audio->dsp_mutex);
+    }
+    while (FALSE);
+
+    if(*buf) {
+      GST_BUFFER_DURATION(*buf) = recv_frames * dsp->frametimemillis * GST_MSECOND;
+      GST_BUFFER_TIMESTAMP(*buf) = dsp->framecount * dsp->frametimemillis * GST_MSECOND;
+      gst_buffer_set_caps (*buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
+      dsp->framecount += recv_frames;
+      return GST_FLOW_OK;
+    }
+    return GST_FLOW_WRONG_STATE;
   }
-  return GST_FLOW_WRONG_STATE;
 }
 
 
@@ -429,6 +525,7 @@ gst_dsppcmsrc_start (GstBaseSrc * bsrc)
 
   // This is used for clocking
   dsp->framecount = 0;
+  dsp->lastcn = NULL;
   DBG_PRINT("gst_dsppcmsrc_start OK\n");
   return TRUE;
 }
@@ -439,13 +536,14 @@ gst_dsppcmsrc_start (GstBaseSrc * bsrc)
  *
  *
  */
-
 static gboolean
 gst_dsppcmsrc_stop (GstBaseSrc * bsrc)
 {
   DBG_PRINT("gst_dsppcmsrc_stop\n");
   GstDSPPCMSrc *dsp = GST_DSPPCMSRC (bsrc);
 
+  gst_dspaudio_remove_interrupt(dsp->audio);
+
   g_mutex_lock(dsp->audio->dsp_mutex);
 
   if ( dsp->audio->all_frames_read ||
@@ -456,8 +554,10 @@ gst_dsppcmsrc_stop (GstBaseSrc * bsrc)
     DBG_PRINT("Got it\n");
   }
 
+  gst_dspaudio_dtmf_tone_stop(dsp->audio);
+
   if(gst_dspaudio_stop(dsp->audio)) {
-    gst_dspaudio_read_cmd(dsp->audio, 0);
+    gst_dspaudio_read_cmd(dsp->audio, 500);
   }
 
   gst_dspaudio_ed_microphone(dsp->audio, FALSE);
@@ -480,7 +580,6 @@ gst_dsppcmsrc_stop (GstBaseSrc * bsrc)
  *
  * Stores the property value into GObject
  */
-
 static void
 gst_dsppcmsrc_set_property(GObject * object,
                            guint prop_id,
@@ -508,7 +607,6 @@ gst_dsppcmsrc_set_property(GObject * object,
  *
  * Retrieves the desired parameter value from object
  */
-
 static void
 gst_dsppcmsrc_get_property(GObject * object,
                            guint prop_id,
@@ -527,11 +625,35 @@ gst_dsppcmsrc_get_property(GObject * object,
 }
 
 
+/*
+ * gst_dsppcmsrc_event:
+ * @src: GstBaseSrc object
+ * @event: GstEvent to be handled
+ *
+ * Handles incoming events. Checks if the event is special "custom upstream
+ * event" and in this case forwards it to dspaudio library for checking if
+ * it is DTMF event.
+ *
+ * Returns: TRUE if the event was handled.
+ */
+static gboolean
+gst_dsppcmsrc_event(GstBaseSrc *src, GstEvent *event)
+{
+  GstEventType type = event ? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;
+  GstDSPPCMSrc *dsp = GST_DSPPCMSRC (src);
+  gboolean ret = FALSE;
+
+  if(type == GST_EVENT_CUSTOM_UPSTREAM) {
+    ret = gst_dspaudio_check_upstream_event (dsp->audio, event);
+  }
+  return ret;
+}
+
+
 /**
  *
  *
  */
-
 static GstClock *
 gst_dsppcmsrc_provide_clock (GstElement * elem)
 {
@@ -545,7 +667,6 @@ gst_dsppcmsrc_provide_clock (GstElement * elem)
  *
  *
  */
-
 static GstClockTime
 gst_dsppcmsrc_get_time (GstClock * clock, GstDSPPCMSrc * src)
 {
@@ -559,12 +680,10 @@ gst_dsppcmsrc_get_time (GstClock * clock, GstDSPPCMSrc * src)
  *
  *
  */
-
 static GstStateChangeReturn
 gst_dsppcmsrc_change_state (GstElement * element, GstStateChange transition)
 {
   GstDSPPCMSrc *dsp = GST_DSPPCMSRC (element);
-  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -576,17 +695,42 @@ gst_dsppcmsrc_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-    switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      gst_dspaudio_remove_interrupt(dsp->audio);
-      break;
-
-    default:
-      break;
-  }
-
-  return ret;
+  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 }
 
+
+/*
+ * gst_dsppcmsrc_request_new_pad:
+ *
+ */
+static GstPad *
+gst_dsppcmsrc_request_new_pad (GstElement     *element,
+                               GstPadTemplate *templ,
+                               const gchar    *name)
+{
+  GstDSPPCMSrc *dsp = GST_DSPPCMSRC(element);
+
+  DBG_PRINT ("request pad\n");
+
+  if (dsp->cnpad)
+    return NULL;
+
+  dsp->cnpad = gst_pad_new_from_template (templ, name);
+  gst_pad_set_activatepush_function (dsp->cnpad, gst_dspaudio_activate_push);
+  gst_element_add_pad (element, dsp->cnpad);
+  return dsp->cnpad;
+}
+
+
+/*
+ * gst_dsppcmsrc_release_pad:
+ *
+ */
+static void
+gst_dsppcmsrc_release_pad (GstElement *element, GstPad *pad)
+{
+  GstDSPPCMSrc *dsp = GST_DSPPCMSRC(element);
+
+  if (pad == dsp->cnpad)
+    dsp->cnpad = NULL;
+}
